@@ -10,8 +10,6 @@
 #include <KTextEditor/Plugin>
 #include <KTextEditor/View>
 
-#include <QVariantList>
-
 class CEPlugin : public KTextEditor::Plugin
 {
     Q_OBJECT
@@ -42,8 +40,6 @@ public:
      */
     explicit CEPluginView(CEPlugin *plugin, KTextEditor::MainWindow *mainwindow);
 
-    Q_SIGNAL void message(const QVariantMap &message);
-
     /**
      * Our Destructor
      */
