@@ -4,6 +4,7 @@
 #include "ce_plugin.h"
 #include "ce_service.h"
 #include "compiledbreader.h"
+#include "ktexteditor_utils.h"
 
 #include <QComboBox>
 #include <QEvent>
@@ -17,8 +18,6 @@
 #include <QMenu>
 #include <QPushButton>
 #include <QSplitter>
-#include <QStandardItemModel>
-#include <QStringListModel>
 #include <QToolButton>
 #include <QTreeView>
 
@@ -26,9 +25,9 @@
 #include <KLocalizedString>
 #include <KMessageBox>
 #include <KSharedConfig>
-#include <KTextEditor/ConfigInterface>
 #include <KTextEditor/MainWindow>
 #include <KXMLGUIFactory>
+#include <kwidgetsaddons_version.h>
 
 enum CE_Options {
     CE_Option_FilterLabel = 1,
@@ -123,8 +122,21 @@ CEWidget::~CEWidget()
 
 bool CEWidget::shouldClose()
 {
-    int ret = KMessageBox::warningYesNo(this, i18n("Do you really want to close %1?", windowTitle()));
+    int ret =
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+        KMessageBox::warningTwoActions(this,
+#else
+        KMessageBox::warningYesNo(this,
+#endif
+                                       i18n("Do you really want to close %1?", windowTitle()),
+                                       {},
+                                       KStandardGuiItem::close(),
+                                       KStandardGuiItem::cancel());
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+    return ret == KMessageBox::PrimaryAction;
+#else
     return ret == KMessageBox::Yes;
+#endif
 }
 
 void CEWidget::removeViewAsActiveXMLGuiClient()
@@ -568,7 +580,7 @@ void CEWidget::sendMessage(const QString &plainText, bool warn)
     genericMessage.insert(QStringLiteral("type"), warn ? QStringLiteral("Error") : QStringLiteral("Info"));
     genericMessage.insert(QStringLiteral("category"), i18n("CompilerExplorer"));
     genericMessage.insert(QStringLiteral("text"), plainText);
-    Q_EMIT m_pluginView->message(genericMessage);
+    Utils::showMessage(genericMessage, m_mainWindow);
 }
 
 void CEWidget::warnIfBadArgs(const QStringList &args)
