@@ -13,7 +13,9 @@
 
 #include <ktexteditor/mainwindow.h>
 
+#include <KDirOperator>
 #include <KFile>
+#include <kio_version.h>
 
 #include <QMenu>
 #include <QUrl>
@@ -64,6 +66,10 @@ public:
         return m_actionCollection;
     }
 
+#if KIO_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+    static KDirOperator::Action actionFromName(const QString &name);
+#endif
+
 public Q_SLOTS:
     void slotFilterChange(const QString &);
     void setDir(const QUrl &);
