@@ -8,10 +8,12 @@
 // BEGIN Includes
 #include "katefiletree.h"
 
-#include "katefiletreedebug.h"
+#include "filehistorywidget.h"
 #include "katefiletreemodel.h"
 #include "katefiletreeproxymodel.h"
 
+#include "katefileactions.h"
+
 #include <ktexteditor/application.h>
 #include <ktexteditor/document.h>
 #include <ktexteditor/editor.h>
@@ -20,7 +22,12 @@
 #include <KIO/ApplicationLauncherJob>
 #include <KIO/CopyJob>
 #include <KIO/DeleteJob>
+#include <kio_version.h>
+#if KIO_VERSION >= QT_VERSION_CHECK(5, 98, 0)
+#include <KIO/JobUiDelegateFactory>
+#else
 #include <KIO/JobUiDelegate>
+#endif
 #include <KIO/OpenFileManagerWindowJob>
 #include <KLocalizedString>
 #include <KMessageBox>
@@ -39,25 +46,27 @@
 #include <QStyledItemDelegate>
 // END Includes
 
-static KTextEditor::Document *docFromIndex(const QModelIndex &index)
+namespace
+{
+KTextEditor::Document *docFromIndex(const QModelIndex &index)
 {
     return index.data(KateFileTreeModel::DocumentRole).value<KTextEditor::Document *>();
 }
 
-static QList<KTextEditor::Document *> docTreeFromIndex(const QModelIndex &index)
+QList<KTextEditor::Document *> docTreeFromIndex(const QModelIndex &index)
 {
     return index.data(KateFileTreeModel::DocumentTreeRole).value<QList<KTextEditor::Document *>>();
 }
 
-static bool closeDocs(const QList<KTextEditor::Document *> &docs)
+bool closeDocs(const QList<KTextEditor::Document *> &docs)
 {
     return KTextEditor::Editor::instance()->application()->closeDocuments(docs);
 }
 
-class StyleDelegate : public QStyledItemDelegate
+class CloseIconStyleDelegate : public QStyledItemDelegate
 {
 public:
-    StyleDelegate(QObject *parent = nullptr)
+    CloseIconStyleDelegate(QObject *parent = nullptr)
         : QStyledItemDelegate(parent)
     {
     }
@@ -66,11 +75,11 @@ public:
     {
         QStyledItemDelegate::paint(painter, option, index);
 
-        if (!m_closeBtn) {
+        if (!m_showCloseBtn) {
             return;
         }
 
-        if (index.column() == 1 && option.state & QStyle::State_MouseOver) {
+        if (index.column() == 1 && option.state & QStyle::State_Enabled && option.state & QStyle::State_MouseOver) {
             const QIcon icon = QIcon::fromTheme(QStringLiteral("tab-close"));
             const int w = option.decorationSize.width();
             QRect iconRect(option.rect.right() - w, option.rect.top(), w, option.rect.height());
@@ -80,29 +89,36 @@ public:
 
     void setShowCloseButton(bool s)
     {
-        m_closeBtn = s;
+        m_showCloseBtn = s;
     }
 
 private:
-    bool m_closeBtn = false;
+    bool m_showCloseBtn = false;
 };
+} // namespace
 
 // BEGIN KateFileTree
 
 KateFileTree::KateFileTree(QWidget *parent)
     : QTreeView(parent)
 {
-    setAcceptDrops(false);
     setIndentation(12);
     setAllColumnsShowFocus(true);
     setFocusPolicy(Qt::NoFocus);
-    setDragEnabled(true);
-    setDragDropMode(QAbstractItemView::DragOnly);
     setSelectionBehavior(QAbstractItemView::SelectRows);
     // for hover close button
     viewport()->setAttribute(Qt::WA_Hover);
 
-    setItemDelegate(new StyleDelegate(this));
+    // DND
+    setDefaultDropAction(Qt::MoveAction);
+    setDragDropMode(QAbstractItemView::InternalMove);
+    setDragDropOverwriteMode(false);
+    setAcceptDrops(true);
+    setDropIndicatorShown(true);
+    setDragEnabled(true);
+    setUniformRowHeights(true);
+
+    setItemDelegate(new CloseIconStyleDelegate(this));
 
     // handle activated (e.g. for pressing enter) + clicked (to avoid to need to do double-click e.g. on Windows)
     connect(this, &KateFileTree::activated, this, &KateFileTree::mouseClicked);
@@ -116,11 +132,11 @@ KateFileTree::KateFileTree(QWidget *parent)
     connect(m_filelistCloseDocument, &QAction::triggered, this, &KateFileTree::slotDocumentClose);
     m_filelistCloseDocument->setWhatsThis(i18n("Close the current document."));
 
-    m_filelistExpandRecursive = new QAction(QIcon::fromTheme(QStringLiteral("view-list-tree")), i18nc("@action:inmenu", "Expand recursively"), this);
+    m_filelistExpandRecursive = new QAction(QIcon::fromTheme(QStringLiteral("view-list-tree")), i18nc("@action:inmenu", "Expand Recursively"), this);
     connect(m_filelistExpandRecursive, &QAction::triggered, this, &KateFileTree::slotExpandRecursive);
     m_filelistExpandRecursive->setWhatsThis(i18n("Expand the file list sub tree recursively."));
 
-    m_filelistCollapseRecursive = new QAction(QIcon::fromTheme(QStringLiteral("view-list-tree")), i18nc("@action:inmenu", "Collapse recursively"), this);
+    m_filelistCollapseRecursive = new QAction(QIcon::fromTheme(QStringLiteral("view-list-tree")), i18nc("@action:inmenu", "Collapse Recursively"), this);
     connect(m_filelistCollapseRecursive, &QAction::triggered, this, &KateFileTree::slotCollapseRecursive);
     m_filelistCollapseRecursive->setWhatsThis(i18n("Collapse the file list sub tree recursively."));
 
@@ -201,7 +217,7 @@ void KateFileTree::onRowsMoved(const QModelIndex &, int, int, const QModelIndex
 void KateFileTree::setShowCloseButton(bool show)
 {
     m_hasCloseButton = show;
-    static_cast<StyleDelegate *>(itemDelegate())->setShowCloseButton(show);
+    static_cast<CloseIconStyleDelegate *>(itemDelegate())->setShowCloseButton(show);
 
     if (!header())
         return;
@@ -246,6 +262,13 @@ void KateFileTree::setupContextMenuActionGroups()
                                        i18nc("@action:inmenu sorting option", "Opening Order"),
                                        i18n("Sort by Opening Order"),
                                        &KateFileTree::slotSortOpeningOrder);
+
+    m_customSorting = new QAction(QIcon(), i18n("Custom Sorting"), this);
+    m_customSorting->setCheckable(true);
+    m_customSorting->setActionGroup(sortGroup);
+    connect(m_customSorting, &QAction::triggered, this, [this] {
+        Q_EMIT sortRoleChanged(CustomSorting);
+    });
 }
 
 QAction *KateFileTree::setupOption(QActionGroup *group,
@@ -302,32 +325,53 @@ void KateFileTree::slotCurrentChanged(const QModelIndex &current, const QModelIn
     }
 }
 
+void KateFileTree::closeClicked(const QModelIndex &index)
+{
+    if (m_proxyModel->isDir(index)) {
+        const QList<KTextEditor::Document *> list = m_proxyModel->docTreeFromIndex(index);
+        closeDocs(list);
+        return;
+    } else if (m_proxyModel->isWidgetDir(index)) {
+        const auto idx = index.siblingAtColumn(0);
+        const auto count = m_proxyModel->rowCount(idx);
+        QWidgetList widgets;
+        widgets.reserve(count);
+        for (int i = 0; i < count; ++i) {
+            widgets << m_proxyModel->index(i, 0, idx).data(KateFileTreeModel::WidgetRole).value<QWidget *>();
+        }
+
+        for (const auto &w : widgets) {
+            closeWidget(w);
+        }
+    }
+
+    if (auto *doc = m_proxyModel->docFromIndex(index)) {
+        closeDocs({doc});
+    } else if (auto *w = index.data(KateFileTreeModel::WidgetRole).value<QWidget *>()) {
+        Q_EMIT closeWidget(w);
+    }
+}
+
 void KateFileTree::mouseClicked(const QModelIndex &index)
 {
-    const bool closeButtonClicked = m_hasCloseButton && index.column() == 1;
-
-    if (m_proxyModel->isDir(index)) {
-        if (closeButtonClicked) {
-            const QList<KTextEditor::Document *> list = m_proxyModel->docTreeFromIndex(index);
-            closeDocs(list);
-        }
+    if (m_hasCloseButton && index.column() == 1) {
+        closeClicked(index);
         return;
     }
 
     if (auto *doc = m_proxyModel->docFromIndex(index)) {
-        if (closeButtonClicked) {
-            closeDocs({doc});
-            return;
-        }
         Q_EMIT activateDocument(doc);
+    } else if (auto *w = index.data(KateFileTreeModel::WidgetRole).value<QWidget *>()) {
+        Q_EMIT activateWidget(w);
     }
 }
 
 void KateFileTree::contextMenuEvent(QContextMenuEvent *event)
 {
-    m_indexContextMenu = selectionModel()->currentIndex();
-
-    selectionModel()->setCurrentIndex(m_indexContextMenu, QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows);
+    m_indexContextMenu = indexAt(event->pos());
+    if (m_indexContextMenu.isValid()) {
+        selectionModel()->setCurrentIndex(m_indexContextMenu, QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows);
+    }
 
     const bool listMode = m_sourceModel->listMode();
     m_treeModeAction->setChecked(!listMode);
@@ -337,43 +381,60 @@ void KateFileTree::contextMenuEvent(QContextMenuEvent *event)
     m_sortByFile->setChecked(sortRole == Qt::DisplayRole);
     m_sortByPath->setChecked(sortRole == KateFileTreeModel::PathRole);
     m_sortByOpeningOrder->setChecked(sortRole == KateFileTreeModel::OpeningOrderRole);
+    m_customSorting->setChecked(sortRole == CustomSorting);
 
     KTextEditor::Document *doc = docFromIndex(m_indexContextMenu);
-    const bool isFile = (nullptr != doc);
 
-    QMenu menu;
-    if (isFile) {
-        QMenu *openWithMenu = menu.addMenu(i18nc("@action:inmenu", "Open With"));
-        openWithMenu->setIcon(QIcon::fromTheme(QStringLiteral("system-run")));
-        connect(openWithMenu, &QMenu::aboutToShow, this, [this, openWithMenu]() {
-            slotFixOpenWithMenu(openWithMenu);
-        });
-        connect(openWithMenu, &QMenu::triggered, this, &KateFileTree::slotOpenWithMenuAction);
+    bool isDir = m_proxyModel->isDir(m_indexContextMenu);
+    bool isWidgetDir = m_proxyModel->isWidgetDir(m_indexContextMenu);
+    bool isWidget = m_indexContextMenu.data(KateFileTreeModel::WidgetRole).value<QWidget *>() != nullptr;
 
-        menu.addSeparator();
-        menu.addAction(m_filelistCopyFilename);
-        menu.addAction(m_filelistRenameFile);
-        menu.addAction(m_filelistDeleteDocument);
-        menu.addAction(m_filelistReloadDocument);
+    QMenu menu(this);
+    if (doc) {
+        if (doc->url().isValid()) {
+            QMenu *openWithMenu = menu.addMenu(i18nc("@action:inmenu", "Open With"));
+            openWithMenu->setIcon(QIcon::fromTheme(QStringLiteral("system-run")));
+            connect(openWithMenu, &QMenu::aboutToShow, this, [this, openWithMenu]() {
+                slotFixOpenWithMenu(openWithMenu);
+            });
+            connect(openWithMenu, &QMenu::triggered, this, &KateFileTree::slotOpenWithMenuAction);
 
-        menu.addSeparator();
-        menu.addAction(m_filelistOpenContainingFolder);
+            menu.addSeparator();
+            menu.addAction(m_filelistCopyFilename);
+            menu.addAction(m_filelistRenameFile);
+            menu.addAction(m_filelistDeleteDocument);
+            menu.addAction(m_filelistReloadDocument);
 
-        menu.addSeparator();
-        menu.addAction(m_filelistCloseDocument);
-        menu.addAction(m_filelistCloseOtherDocument);
+            if (doc->url().isLocalFile()) {
+                auto a = menu.addAction(i18n("Show File Git History"));
+                connect(a, &QAction::triggered, this, [doc] {
+                    auto url = doc->url();
+                    if (url.isValid() && url.isLocalFile()) {
+                        FileHistory::showFileHistory(url.toLocalFile());
+                    }
+                });
+            }
 
-        menu.addSeparator();
-        menu.addAction(m_filelistPrintDocument);
-        menu.addAction(m_filelistPrintDocumentPreview);
+            menu.addSeparator();
+            menu.addAction(m_filelistOpenContainingFolder);
 
-        const bool hasFileName = doc->url().isValid();
-        m_filelistOpenContainingFolder->setEnabled(hasFileName);
-        m_filelistCopyFilename->setEnabled(hasFileName);
-        m_filelistRenameFile->setEnabled(hasFileName);
-        m_filelistDeleteDocument->setEnabled(hasFileName);
-    } else {
-        menu.addAction(m_filelistReloadDocument);
+            menu.addSeparator();
+            menu.addAction(m_filelistCloseDocument);
+            menu.addAction(m_filelistCloseOtherDocument);
+
+            menu.addSeparator();
+            menu.addAction(m_filelistPrintDocument);
+            menu.addAction(m_filelistPrintDocumentPreview);
+        } else {
+            // untitled documents
+            menu.addAction(m_filelistCloseDocument);
+
+            menu.addSeparator();
+        }
+    } else if (isDir || isWidgetDir || isWidget) {
+        if (isDir) {
+            menu.addAction(m_filelistReloadDocument);
+        }
 
         menu.addSeparator();
         menu.addAction(m_filelistCloseDocument);
@@ -392,6 +453,9 @@ void KateFileTree::contextMenuEvent(QContextMenuEvent *event)
     sort_menu->addAction(m_sortByFile);
     sort_menu->addAction(m_sortByPath);
     sort_menu->addAction(m_sortByOpeningOrder);
+    sort_menu->addAction(m_customSorting);
+
+    m_filelistCloseDocument->setEnabled(m_indexContextMenu.isValid());
 
     menu.addAction(m_resetHistory);
 
@@ -444,17 +508,22 @@ void KateFileTree::slotOpenWithMenuAction(QAction *a)
     // If app is null, ApplicationLauncherJob will invoke the open-with dialog
     auto *job = new KIO::ApplicationLauncherJob(app);
     job->setUrls(list);
+#if KIO_VERSION >= QT_VERSION_CHECK(5, 98, 0)
+    job->setUiDelegate(KIO::createDefaultJobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, this));
+#else
     job->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, this));
+#endif
     job->start();
 }
 
-Q_DECLARE_METATYPE(QList<KTextEditor::Document *>)
-
 void KateFileTree::slotDocumentClose()
 {
     m_previouslySelected = QModelIndex();
-    const QList<KTextEditor::Document *> closingDocuments = docTreeFromIndex(m_indexContextMenu);
-    closeDocs(closingDocuments);
+    if (!m_indexContextMenu.isValid()) {
+        return;
+    }
+    const auto closeColumnIndex = m_indexContextMenu.sibling(m_indexContextMenu.row(), 1);
+    closeClicked(closeColumnIndex);
 }
 
 void KateFileTree::addChildrenTolist(const QModelIndex &index, QList<QPersistentModelIndex> *worklist)
@@ -546,49 +615,7 @@ void KateFileTree::slotCopyFilename()
 
 void KateFileTree::slotRenameFile()
 {
-    KTextEditor::Document *doc = m_proxyModel->docFromIndex(m_indexContextMenu);
-
-    // TODO: the following code was improved in kate/katefileactions.cpp and should be reused here
-
-    if (!doc) {
-        return;
-    }
-
-    const QUrl oldFileUrl = doc->url();
-    const QString oldFileName = doc->url().fileName();
-    bool ok;
-
-    QString newFileName = QInputDialog::getText(this, i18n("Rename file"), i18n("New file name"), QLineEdit::Normal, oldFileName, &ok);
-    if (!ok) {
-        return;
-    }
-
-    QUrl newFileUrl = oldFileUrl.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash);
-    newFileUrl.setPath(newFileUrl.path() + QLatin1Char('/') + newFileName);
-
-    if (!newFileUrl.isValid()) {
-        return;
-    }
-
-    if (!doc->closeUrl()) {
-        return;
-    }
-
-    doc->waitSaveComplete();
-
-    KIO::CopyJob *job = KIO::move(oldFileUrl, newFileUrl);
-    QSharedPointer<QMetaObject::Connection> sc(new QMetaObject::Connection());
-    auto success = [doc, sc](KIO::Job *, const QUrl &, const QUrl &realNewFileUrl, const QDateTime &, bool, bool) {
-        doc->openUrl(realNewFileUrl);
-        doc->documentSavedOrUploaded(doc, true);
-        QObject::disconnect(*sc);
-    };
-    *sc = connect(job, &KIO::CopyJob::copyingDone, doc, success);
-
-    if (!job->exec()) {
-        KMessageBox::error(this, i18n("File \"%1\" could not be moved to \"%2\"", oldFileUrl.toDisplayString(), newFileUrl.toDisplayString()));
-        doc->openUrl(oldFileUrl);
-    }
+    KateFileActions::renameDocumentFile(this, m_proxyModel->docFromIndex(m_indexContextMenu));
 }
 
 void KateFileTree::slotDocumentFirst()
@@ -675,8 +702,11 @@ void KateFileTree::slotDocumentPrev()
     }
 
     if (prev.isValid()) {
-        KTextEditor::Document *doc = m_proxyModel->docFromIndex(prev);
-        Q_EMIT activateDocument(doc);
+        if (auto *doc = m_proxyModel->docFromIndex(prev)) {
+            Q_EMIT activateDocument(doc);
+        } else if (auto *w = prev.data(KateFileTreeModel::WidgetRole).value<QWidget *>()) {
+            Q_EMIT activateWidget(w);
+        }
     }
 }
 
@@ -747,8 +777,11 @@ void KateFileTree::slotDocumentNext()
     }
 
     if (next.isValid()) {
-        KTextEditor::Document *doc = m_proxyModel->docFromIndex(next);
-        Q_EMIT activateDocument(doc);
+        if (auto *doc = m_proxyModel->docFromIndex(next)) {
+            Q_EMIT activateDocument(doc);
+        } else if (auto *w = next.data(KateFileTreeModel::WidgetRole).value<QWidget *>()) {
+            Q_EMIT activateWidget(w);
+        }
     }
 }
 
