@@ -62,6 +62,9 @@ Q_SIGNALS:
     void viewModeChanged(bool treeMode);
     void sortRoleChanged(int);
 
+    void closeWidget(QWidget *);
+    void activateWidget(QWidget *);
+
 private Q_SLOTS:
     void mouseClicked(const QModelIndex &index);
 
@@ -79,6 +82,7 @@ private Q_SLOTS:
     void onRowsMoved(const QModelIndex &parent, int start, int end, const QModelIndex &destination, int row);
 
 private:
+    void closeClicked(const QModelIndex &index);
     void setupContextMenuActionGroups();
     using Func = void (KateFileTree::*)();
     QAction *setupOption(QActionGroup *group,
@@ -108,6 +112,7 @@ private:
     QAction *m_sortByFile;
     QAction *m_sortByPath;
     QAction *m_sortByOpeningOrder;
+    QAction *m_customSorting;
     QAction *m_resetHistory;
 
     KateFileTreeProxyModel *m_proxyModel = nullptr;
