@@ -30,7 +30,13 @@ class KateFileTreeModel : public QAbstractItemModel
     Q_OBJECT
 
 public:
-    enum { DocumentRole = Qt::UserRole + 1, PathRole, OpeningOrderRole, DocumentTreeRole };
+    enum {
+        DocumentRole = Qt::UserRole + 1,
+        PathRole,
+        OpeningOrderRole,
+        DocumentTreeRole,
+        WidgetRole,
+    };
 
     KateFileTreeModel(QObject *p);
     ~KateFileTreeModel() override;
@@ -48,10 +54,15 @@ public:
 
     QMimeData *mimeData(const QModelIndexList &indexes) const override;
 
+    Qt::DropActions supportedDropActions() const override;
+
     /* extra api for view */
     QModelIndex docIndex(const KTextEditor::Document *) const;
 
-    static bool isDir(const QModelIndex &index);
+    QModelIndex widgetIndex(QWidget *) const;
+
+    bool isDir(const QModelIndex &index) const;
+    bool isWidgetDir(const QModelIndex &index) const;
 
     bool listMode() const;
     void setListMode(bool);
@@ -71,6 +82,9 @@ public:
     void documentEdited(const KTextEditor::Document *);
     void resetHistory();
 
+    bool dropMimeData(const QMimeData *data, Qt::DropAction action, int row, int column, const QModelIndex &parent) override;
+    bool canDropMimeData(const QMimeData *data, Qt::DropAction action, int row, int column, const QModelIndex &parent) const override;
+
 public Q_SLOTS:
     void documentOpened(KTextEditor::Document *);
     void documentClosed(KTextEditor::Document *);
@@ -78,6 +92,9 @@ public Q_SLOTS:
     void documentModifiedChanged(KTextEditor::Document *);
     void documentModifiedOnDisc(KTextEditor::Document *, bool, KTextEditor::ModificationInterface::ModifiedOnDiskReason);
 
+    void addWidget(QWidget *w);
+    void removeWidget(QWidget *w);
+
 Q_SIGNALS:
     void triggerViewChangeAfterNameChange();
 
@@ -97,9 +114,11 @@ private:
     void initModel();
     void clearModel();
     void connectDocument(const KTextEditor::Document *);
+    ProxyItem *itemForIndex(const QModelIndex &index) const;
 
 private:
     ProxyItemDir *m_root;
+    ProxyItem *m_widgetsRoot = nullptr;
     QHash<const KTextEditor::Document *, ProxyItem *> m_docmap;
 
     bool m_shadingEnabled;
@@ -115,4 +134,6 @@ private:
     bool m_listMode;
 };
 
+Q_DECLARE_METATYPE(QList<KTextEditor::Document *>)
+
 #endif /* KATEFILETREEMODEL_H */
