@@ -15,7 +15,6 @@
 #include <ktexteditor/application.h>
 #include <ktexteditor/view.h>
 
-#include <KAboutData>
 #include <KActionCollection>
 #include <KConfigGroup>
 #include <KLocalizedString>
@@ -24,7 +23,6 @@
 #include <KXmlGuiWindow>
 
 #include <QAction>
-#include <QApplication>
 #include <QLayout>
 #include <QLineEdit>
 #include <QStyle>
@@ -113,6 +111,7 @@ void KateFileTreePlugin::applyConfig(bool shadingEnabled,
         view->model()->setEditShade(editShade);
         view->setListMode(listMode);
         view->proxy()->setSortRole(sortRole);
+        view->tree()->setDragDropMode(sortRole == CustomSorting ? QAbstractItemView::InternalMove : QAbstractItemView::DragOnly);
         view->model()->setShowFullPathOnRoots(showFullPath);
         view->setToolbarVisible(showToolbar);
         view->tree()->setShowCloseButton(showCloseButton);
@@ -127,13 +126,13 @@ KateFileTreePluginView::KateFileTreePluginView(KTextEditor::MainWindow *mainWind
     , m_plug(plug)
     , m_mainWindow(mainWindow)
 {
-    KXMLGUIClient::setComponentName(QStringLiteral("katefiletree"), i18n("Kate File Tree"));
+    KXMLGUIClient::setComponentName(QStringLiteral("katefiletree"), i18n("Documents"));
     setXMLFile(QStringLiteral("ui.rc"));
 
     m_toolView = mainWindow->createToolView(plug,
                                             QStringLiteral("kate_private_plugin_katefiletreeplugin"),
                                             KTextEditor::MainWindow::Left,
-                                            QIcon::fromTheme(QStringLiteral("document-open")),
+                                            QIcon::fromTheme(QStringLiteral("folder-documents-symbolic")),
                                             i18n("Documents"));
 
     // create toolbar
@@ -195,17 +194,25 @@ KateFileTreePluginView::KateFileTreePluginView(KTextEditor::MainWindow *mainWind
     });
 
     m_fileTree->setModel(m_proxyModel);
-
-    m_fileTree->setDragEnabled(false);
-    m_fileTree->setDragDropMode(QAbstractItemView::InternalMove);
-    m_fileTree->setDropIndicatorShown(false);
-
     m_fileTree->setSelectionMode(QAbstractItemView::SingleSelection);
 
     connect(m_fileTree->selectionModel(), &QItemSelectionModel::currentChanged, m_fileTree, &KateFileTree::slotCurrentChanged);
 
     connect(mainWindow, &KTextEditor::MainWindow::viewChanged, this, &KateFileTreePluginView::viewChanged);
 
+    auto mw = mainWindow->window();
+    connect(mw, SIGNAL(widgetAdded(QWidget *)), this, SLOT(slotWidgetCreated(QWidget *)));
+    connect(mw, SIGNAL(widgetRemoved(QWidget *)), this, SLOT(slotWidgetRemoved(QWidget *)));
+
+    connect(m_fileTree, &KateFileTree::closeWidget, this, [this](QWidget *w) {
+        auto mw = m_mainWindow->window();
+        QMetaObject::invokeMethod(mw, "removeWidget", Q_ARG(QWidget *, w));
+    });
+    connect(m_fileTree, &KateFileTree::activateWidget, this, [this](QWidget *w) {
+        auto mw = m_mainWindow->window();
+        QMetaObject::invokeMethod(mw, "activateWidget", Q_ARG(QWidget *, w));
+    });
+
     //
     // actions
     //
@@ -216,6 +223,7 @@ KateFileTreePluginView::KateFileTreePluginView(KTextEditor::MainWindow *mainWind
     setToolbarVisible(m_plug->settings().showToolbar());
 
     m_proxyModel->setSortRole(Qt::DisplayRole);
+    m_fileTree->setDragDropMode(QAbstractItemView::DragOnly);
 
     m_proxyModel->sort(0, Qt::AscendingOrder);
     m_proxyModel->invalidate();
@@ -247,16 +255,18 @@ void KateFileTreePluginView::setupActions()
     connect(aNext, &QAction::triggered, m_fileTree, &KateFileTree::slotDocumentNext);
 
     auto aShowActive = actionCollection()->addAction(QStringLiteral("filetree_show_active_document"));
-    aShowActive->setText(i18n("&Show Active"));
+    aShowActive->setText(i18n("&Show Active Document"));
     aShowActive->setIcon(QIcon::fromTheme(QStringLiteral("folder-sync")));
     connect(aShowActive, &QAction::triggered, this, &KateFileTreePluginView::showActiveDocument);
 
     auto aSave = actionCollection()->addAction(QStringLiteral("filetree_save"), this, SLOT(slotDocumentSave()));
+    aSave->setText(i18n("Save"));
     aSave->setToolTip(i18n("Save the current document"));
     aSave->setIcon(QIcon::fromTheme(QStringLiteral("document-save")));
 
     auto aSaveAs = actionCollection()->addAction(QStringLiteral("filetree_save_as"), this, SLOT(slotDocumentSaveAs()));
-    aSaveAs->setToolTip(i18n("Save current document under new name"));
+    aSaveAs->setText(i18n("Save As"));
+    aSaveAs->setToolTip(i18n("Save the current document under a new name"));
     aSaveAs->setIcon(QIcon::fromTheme(QStringLiteral("document-save-as")));
 
     /**
@@ -280,11 +290,12 @@ void KateFileTreePluginView::setupActions()
     /**
      * add own actions
      */
-    m_toolbar->addAction(aPrev);
-    m_toolbar->addAction(aNext);
-    m_toolbar->addSeparator();
     m_toolbar->addAction(aSave);
     m_toolbar->addAction(aSaveAs);
+    m_toolbar->addSeparator();
+    m_toolbar->addAction(aPrev);
+    m_toolbar->addAction(aNext);
+    m_toolbar->addAction(aShowActive);
 }
 
 KateFileTreeModel *KateFileTreePluginView::model() const
@@ -322,16 +333,22 @@ void KateFileTreePluginView::setToolbarVisible(bool visible)
 
 void KateFileTreePluginView::viewChanged(KTextEditor::View *)
 {
-    KTextEditor::View *view = m_mainWindow->activeView();
-    if (!view) {
+    auto mw = m_mainWindow->window();
+    QWidget *activeWidget = nullptr;
+    QMetaObject::invokeMethod(mw, "activeWidget", Q_RETURN_ARG(QWidget *, activeWidget));
+    if (!activeWidget) {
         return;
     }
 
-    KTextEditor::Document *doc = view->document();
-    QModelIndex index = m_proxyModel->docIndex(doc);
-
-    // update the model on which doc is active
-    m_documentModel->documentActivated(doc);
+    QModelIndex index;
+    if (auto view = qobject_cast<KTextEditor::View *>(activeWidget)) {
+        KTextEditor::Document *doc = view->document();
+        index = m_proxyModel->docIndex(doc);
+        // update the model on which doc is active
+        m_documentModel->documentActivated(doc);
+    } else {
+        index = m_proxyModel->widgetIndex(activeWidget);
+    }
 
     m_fileTree->selectionModel()->setCurrentIndex(index, QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows);
 
@@ -368,6 +385,7 @@ void KateFileTreePluginView::sortRoleChanged(int role)
     setHasLocalPrefs(true);
     m_proxyModel->setSortRole(role);
     m_proxyModel->invalidate();
+    m_fileTree->setDragDropMode(role == CustomSorting ? QAbstractItemView::InternalMove : QAbstractItemView::DragOnly);
 }
 
 void KateFileTreePluginView::activateDocument(KTextEditor::Document *doc)
@@ -422,6 +440,7 @@ void KateFileTreePluginView::readSessionConfig(const KConfigGroup &g)
 
     int sortRole = g.readEntry("sortRole", defaults.sortRole());
     m_proxyModel->setSortRole(sortRole);
+    m_fileTree->setDragDropMode(sortRole == CustomSorting ? QAbstractItemView::InternalMove : QAbstractItemView::DragOnly);
 }
 
 void KateFileTreePluginView::writeSessionConfig(KConfigGroup &g)
@@ -459,6 +478,16 @@ void KateFileTreePluginView::slotDocumentSaveAs() const
     }
 }
 
+void KateFileTreePluginView::slotWidgetCreated(QWidget *w)
+{
+    m_documentModel->addWidget(w);
+}
+
+void KateFileTreePluginView::slotWidgetRemoved(QWidget *w)
+{
+    m_documentModel->removeWidget(w);
+}
+
 // END KateFileTreePluginView
 
 #include "katefiletreeplugin.moc"
