@@ -7,10 +7,8 @@
 #ifndef KATE_FILETREE_PLUGIN_H
 #define KATE_FILETREE_PLUGIN_H
 
-#include <QIcon>
 #include <QTimer>
 
-#include <KTextEditor/Command>
 #include <KTextEditor/Plugin>
 #include <ktexteditor/document.h>
 #include <ktexteditor/editor.h>
@@ -135,6 +133,9 @@ private Q_SLOTS:
     void slotDocumentsCreated();
     void slotDocumentSave() const;
     void slotDocumentSaveAs() const;
+
+    void slotWidgetCreated(QWidget *);
+    void slotWidgetRemoved(QWidget *);
 };
 
 #endif // KATE_FILETREE_PLUGIN_H
