@@ -19,25 +19,17 @@
 #include <QLayout>
 #include <QPushButton>
 #include <QStandardPaths>
-#include <QTimer>
 
 #include <KTextEditor/Document>
 #include <KTextEditor/View>
 
 #include <KLocalizedString>
-#include <KMessageBox>
 
 #include "dap/settings.h"
 #include "json_placeholders.h"
 
 #include <json_utils.h>
 
-#ifdef WIN32
-static const QLatin1Char pathSeparator(';');
-#else
-static const QLatin1Char pathSeparator(':');
-#endif
-
 constexpr int CONFIG_VERSION = 5;
 const static QString F_TARGET = QStringLiteral("target");
 const static QString F_DEBUGGER = QStringLiteral("debuggerKey");
@@ -93,7 +85,7 @@ ConfigView::ConfigView(QWidget *parent, KTextEditor::MainWindow *mainWin)
     m_addTarget->setToolTip(i18n("Add new target"));
 
     m_copyTarget = new QToolButton(this);
-    m_copyTarget->setIcon(QIcon::fromTheme(QStringLiteral("document-copy")));
+    m_copyTarget->setIcon(QIcon::fromTheme(QStringLiteral("edit-copy")));
     m_copyTarget->setToolTip(i18n("Copy target"));
 
     m_deleteTarget = new QToolButton(this);
@@ -131,7 +123,7 @@ ConfigView::ConfigView(QWidget *parent, KTextEditor::MainWindow *mainWin)
     m_processId = new QSpinBox(this);
     m_processId->setMinimum(1);
     m_processId->setMaximum(4194304);
-    m_processIdLabel = new QLabel(i18n("Process Id:"));
+    m_processIdLabel = new QLabel(i18n("Process Id:"), this);
     m_processIdLabel->setBuddy(m_processId);
 
     m_arguments = new QLineEdit(this);
@@ -587,6 +579,7 @@ void ConfigView::resizeEvent(QResizeEvent *)
         m_checBoxLayout = nullptr;
         delete layout();
         QGridLayout *layout = new QGridLayout(this);
+        layout->setContentsMargins(0, 0, 0, 0);
 
         layout->addWidget(m_clientCombo, 0, 0);
         layout->addWidget(m_targetCombo, 1, 0);
@@ -659,6 +652,7 @@ void ConfigView::resizeEvent(QResizeEvent *)
         }
 
         QGridLayout *layout = new QGridLayout(this);
+        layout->setContentsMargins(0, 0, 0, 0);
         layout->addWidget(m_clientCombo, 0, 0, 1, 6);
 
         layout->addWidget(m_targetCombo, 1, 0, 1, 3);
