@@ -13,6 +13,7 @@
 #include "dap/entities.h"
 #include "hostprocess.h"
 
+#include <QDir>
 #include <QFile>
 #include <QFileInfo>
 #include <QRegularExpression>
@@ -21,7 +22,6 @@
 
 #include <KLocalizedString>
 #include <KMessageBox>
-#include <KUrlRequester>
 
 #include <signal.h>
 #include <stdlib.h>
@@ -73,6 +73,12 @@ bool DebugView::canMove() const
     return debuggerRunning();
 }
 
+bool DebugView::canContinue() const
+{
+    // true to preserve the behaviour previous to DAP
+    return true;
+}
+
 void DebugView::runDebugger(const GDBTargetConf &conf, const QStringList &ioFifos)
 {
     if (conf.executable.isEmpty()) {
@@ -163,7 +169,7 @@ void DebugView::toggleBreakpoint(QUrl const &url, int line)
 
 void DebugView::slotError()
 {
-    KMessageBox::sorry(nullptr, i18n("Could not start debugger process"));
+    KMessageBox::error(nullptr, i18n("Could not start debugger process"));
 }
 
 void DebugView::slotReadDebugStdOut()
