@@ -8,6 +8,7 @@
 #ifndef LOCALSVIEW_H
 #define LOCALSVIEW_H
 
+#include "dap/entities.h"
 #include <QTreeWidget>
 #include <QTreeWidgetItem>
 
@@ -20,9 +21,9 @@ public:
 
 public Q_SLOTS:
     // An empty value string ends the locals
-    void addLocal(const QString &vString);
-    void addStruct(QTreeWidgetItem *parent, const QString &vString);
-    void addArray(QTreeWidgetItem *parent, const QString &vString);
+    void openVariableScope();
+    void closeVariableScope();
+    void addVariableLevel(int parentId, const dap::Variable &variable);
 
 Q_SIGNALS:
     void localsVisible(bool visible);
@@ -32,10 +33,10 @@ protected:
     void hideEvent(QHideEvent *event) override;
 
 private:
-    void createWrappedItem(QTreeWidgetItem *parent, const QString &name, const QString &value);
-    void createWrappedItem(QTreeWidget *parent, const QString &name, const QString &value);
-    bool m_allAdded = true;
-    QString m_local;
+    QTreeWidgetItem *createWrappedItem(QTreeWidgetItem *parent, const dap::Variable &variable);
+    QTreeWidgetItem *createWrappedItem(QTreeWidget *parent, const dap::Variable &variable);
+
+    QHash<int, QTreeWidgetItem *> m_variables;
 };
 
 #endif
