@@ -8,13 +8,11 @@
 
 #include <gitprocess.h>
 
-#include <QDebug>
 #include <QDir>
 #include <QMimeDatabase>
 #include <QPainter>
 #include <QProcess>
 #include <QStyledItemDelegate>
-#include <QUrl>
 #include <QVBoxLayout>
 
 #include <KColorScheme>
@@ -260,52 +258,6 @@ static void createFileTree(QStandardItem *parent, const QString &basePath, const
     }
 }
 
-static std::optional<QString> getGitCmdOutput(const QString &workDir, const QStringList &args)
-{
-    QProcess git;
-    if (!setupGitProcess(git, workDir, args)) {
-        return {};
-    }
-    startHostProcess(git, QProcess::ReadOnly);
-    if (git.waitForStarted() && git.waitForFinished(-1)) {
-        if (git.exitStatus() != QProcess::NormalExit || git.exitCode() != 0) {
-            return std::nullopt;
-        }
-        return {QString::fromUtf8(git.readAllStandardOutput().trimmed())};
-    }
-    return {};
-}
-
-static std::optional<QString> getDotGitPath(const QString &repo)
-{
-    /* This call is intentionally blocking because we need git path for everything else */
-    auto dotGitPathRes = getGitCmdOutput(repo, {QStringLiteral("rev-parse"), QStringLiteral("--absolute-git-dir")});
-    if (!dotGitPathRes.has_value()) {
-        return {};
-    }
-    QString dotGitPath = dotGitPathRes.value();
-
-    if (dotGitPath.endsWith(QLatin1String(".git"))) {
-        dotGitPath.remove(QLatin1String(".git"));
-    } else if (dotGitPath.contains(QLatin1String(".git/modules"))) {
-        // maybe this is some submodule
-        auto topLevelRes = getGitCmdOutput(repo, {QStringLiteral("rev-parse"), QStringLiteral("--show-toplevel")});
-        if (!topLevelRes.has_value()) {
-            return {};
-        }
-        QString topLevel = QDir::cleanPath(topLevelRes.value());
-        if (!topLevel.endsWith(QLatin1Char('/'))) {
-            topLevel += QStringLiteral("/");
-        }
-        return topLevel;
-    } else {
-        qWarning() << "[blame] Got invalid dot git path: " << dotGitPath;
-        return {};
-    }
-    return dotGitPath;
-    return std::nullopt;
-}
-
 static bool getNum(const QByteArray &numBytes, int *num)
 {
     bool res = false;
@@ -393,7 +345,7 @@ void CommitDiffTreeView::createFileTreeForCommit(const QString &filePath, const
 {
     QFileInfo fi(filePath);
     QString path = fi.absolutePath();
-    auto value = getDotGitPath(path);
+    auto value = getRepoBasePath(path);
     if (value.has_value()) {
         m_gitDir = value.value();
     }
@@ -456,5 +408,5 @@ void CommitDiffTreeView::showDiff(const QModelIndex &idx)
         }
     }
 
-    Q_EMIT showDiffRequested(git.readAllStandardOutput());
+    Q_EMIT showDiffRequested(git.readAllStandardOutput(), file);
 }
