@@ -7,10 +7,11 @@
 #ifndef GitBlameTooltip_h
 #define GitBlameTooltip_h
 
-#include <QPointer>
+#include <memory>
 
 class QString;
 class KateGitBlamePluginView;
+class QKeySequence;
 
 namespace KTextEditor
 {
@@ -23,13 +24,14 @@ public:
     GitBlameTooltip(KateGitBlamePluginView *pv);
     ~GitBlameTooltip();
 
-    void show(const QString &text, QPointer<KTextEditor::View> view);
+    void show(const QString &text, KTextEditor::View *view);
 
-    void setIgnoreKeySequence(QKeySequence sequence);
+    void setIgnoreKeySequence(const QKeySequence &sequence);
 
 private:
     class Private;
-    Private *const d;
+    std::unique_ptr<Private> d;
+    KateGitBlamePluginView *m_pluginView;
 };
 
 #endif // GitBlameTooltip_h
