@@ -6,13 +6,14 @@
 
 #include "kategitblameplugin.h"
 #include "commitfilesview.h"
+#include "diffparams.h"
+#include "ktexteditor_utils.h"
 
 #include <gitprocess.h>
 
 #include <algorithm>
 
 #include <KActionCollection>
-#include <KConfigGroup>
 #include <KLocalizedString>
 #include <KPluginFactory>
 #include <KSharedConfig>
@@ -27,6 +28,7 @@
 #include <QUrl>
 
 #include <QFontMetrics>
+#include <QKeySequence>
 #include <QLayout>
 #include <QPainter>
 #include <QVariant>
@@ -165,7 +167,7 @@ KateGitBlamePluginView::KateGitBlamePluginView(KateGitBlamePlugin *plugin, KText
     setXMLFile(QStringLiteral("ui.rc"));
     QAction *showBlameAction = actionCollection()->addAction(QStringLiteral("git_blame_show"));
     showBlameAction->setText(i18n("Show Git Blame Details"));
-    actionCollection()->setDefaultShortcut(showBlameAction, Qt::CTRL | Qt::ALT | Qt::Key_G);
+    actionCollection()->setDefaultShortcut(showBlameAction, QKeySequence(QStringLiteral("Ctrl+T, B"), QKeySequence::PortableText));
     QAction *toggleBlameAction = actionCollection()->addAction(QStringLiteral("git_blame_toggle"));
     toggleBlameAction->setText(i18n("Toggle Git Blame Details"));
     m_mainWindow->guiFactory()->addClient(this);
@@ -184,7 +186,13 @@ KateGitBlamePluginView::KateGitBlamePluginView(KateGitBlamePlugin *plugin, KText
         m_inlineNoteProvider.cycleMode();
     });
 
-    connect(m_mainWindow, &KTextEditor::MainWindow::viewChanged, this, &KateGitBlamePluginView::viewChanged);
+    m_startBlameTimer.setSingleShot(true);
+    m_startBlameTimer.setInterval(400);
+    m_startBlameTimer.callOnTimeout(this, &KateGitBlamePluginView::startGitBlameForActiveView);
+
+    connect(m_mainWindow, &KTextEditor::MainWindow::viewChanged, this, [this](KTextEditor::View *) {
+        m_startBlameTimer.start();
+    });
 
     connect(&m_blameInfoProc, static_cast<void (QProcess::*)(int, QProcess::ExitStatus)>(&QProcess::finished), this, &KateGitBlamePluginView::blameFinished);
 
@@ -218,13 +226,14 @@ QPointer<KTextEditor::Document> KateGitBlamePluginView::activeDocument() const
     return nullptr;
 }
 
-void KateGitBlamePluginView::viewChanged(KTextEditor::View *view)
+void KateGitBlamePluginView::startGitBlameForActiveView()
 {
     if (m_lastView) {
         qobject_cast<KTextEditor::InlineNoteInterface *>(m_lastView)->unregisterInlineNoteProvider(&m_inlineNoteProvider);
     }
-    m_lastView = view;
 
+    auto *view = m_mainWindow->activeView();
+    m_lastView = view;
     if (view == nullptr || view->document() == nullptr) {
         return;
     }
@@ -307,9 +316,9 @@ void KateGitBlamePluginView::sendMessage(const QString &text, bool error)
     QVariantMap genericMessage;
     genericMessage.insert(QStringLiteral("type"), error ? QStringLiteral("Error") : QStringLiteral("Info"));
     genericMessage.insert(QStringLiteral("category"), i18n("Git"));
-    genericMessage.insert(QStringLiteral("categoryIcon"), QIcon(QStringLiteral(":/icons/icons/sc-apps-git.svg")));
+    genericMessage.insert(QStringLiteral("categoryIcon"), gitIcon());
     genericMessage.insert(QStringLiteral("text"), text);
-    Q_EMIT message(genericMessage);
+    Utils::showMessage(genericMessage, m_mainWindow);
 }
 
 void KateGitBlamePluginView::blameFinished(int exitCode, QProcess::ExitStatus exitStatus)
@@ -564,11 +573,7 @@ void KateGitBlamePluginView::createToolView()
     }
 
     auto plugin = static_cast<KTextEditor::Plugin *>(parent());
-    m_toolView.reset(m_mainWindow->createToolView(plugin,
-                                                  QStringLiteral("commitfilesview"),
-                                                  KTextEditor::MainWindow::Left,
-                                                  QIcon::fromTheme(QStringLiteral(":/icons/icons/sc-apps-git.svg")),
-                                                  i18n("Commit")));
+    m_toolView.reset(m_mainWindow->createToolView(plugin, QStringLiteral("commitfilesview"), KTextEditor::MainWindow::Left, gitIcon(), i18n("Commit")));
 
     m_commitFilesView = new CommitDiffTreeView(m_toolView.get());
     m_toolView->layout()->addWidget(m_commitFilesView);
@@ -583,21 +588,12 @@ void KateGitBlamePluginView::hideToolView()
     // CommitFileView will be destroyed as well as it is the child of m_ToolView
 }
 
-void KateGitBlamePluginView::showDiffForFile(const QByteArray &diffContents)
+void KateGitBlamePluginView::showDiffForFile(const QByteArray &diffContents, const QString &file)
 {
-    if (m_diffView) {
-        m_diffView->document()->setText(QString::fromUtf8(diffContents));
-        m_diffView->document()->setModified(false);
-        m_mainWindow->activateView(m_diffView->document());
-        m_diffView->setCursorPosition({0, 0});
-        return;
-    }
-    m_diffView = m_mainWindow->openUrl(QUrl());
-    m_diffView->document()->setHighlightingMode(QStringLiteral("Diff"));
-    m_diffView->document()->setText(QString::fromUtf8(diffContents));
-    m_diffView->document()->setModified(false);
-    m_mainWindow->activateView(m_diffView->document());
-    m_diffView->setCursorPosition({0, 0});
+    DiffParams d;
+    d.srcFile = file;
+    d.flags.setFlag(DiffParams::ShowCommitInfo);
+    Utils::showDiff(diffContents, d, m_mainWindow);
 }
 
 #include "kategitblameplugin.moc"
