@@ -1,8 +1,8 @@
 /***************************************************************************
  *   This file is part of Kate build plugin                                *
- *   SPDX-FileCopyrightText: 2014 Kåre Särs <kare.sars@iki.fi>                           *
+ *   SPDX-FileCopyrightText: 2014 Kåre Särs <kare.sars@iki.fi>             *
  *                                                                         *
- *   SPDX-License-Identifier: LGPL-2.0-or-later
+ *   SPDX-License-Identifier: LGPL-2.0-or-later                            *
  ***************************************************************************/
 
 #ifndef TargetModel_h
@@ -15,20 +15,21 @@ class TargetModel : public QAbstractItemModel
 {
     Q_OBJECT
 public:
+    struct Command {
+        QString name;
+        QString buildCmd;
+        QString runCmd;
+    };
     struct TargetSet {
         TargetSet(const QString &_name, const QString &_workDir);
         QString name;
         QString workDir;
-        QString defaultCmd;
-        QList<QPair<QString, QString>> commands;
+        QList<Command> commands;
     };
 
     TargetModel(QObject *parent = nullptr);
     ~TargetModel() override;
 
-    /** This function sets the default command for a target set */
-    void setDefaultCmd(int rootRow, const QString &defCmd);
-
 public Q_SLOTS:
 
     /** This function clears all the target-sets */
@@ -38,8 +39,12 @@ public Q_SLOTS:
      * inserted target-set */
     QModelIndex addTargetSet(const QString &setName, const QString &workDir);
 
+    /** This function insert a target set and returns the model-index of the newly
+     * inserted target-set */
+    QModelIndex insertTargetSet(int row, const QString &setName, const QString &workDir);
+
     /** This function adds a new command to a target-set and returns the model index */
-    QModelIndex addCommand(const QModelIndex &parentIndex, const QString &cmdName, const QString &command);
+    QModelIndex addCommand(const QModelIndex &parentIndex, const QString &cmdName, const QString &buildCmd, const QString &runCmd);
 
     /** This function copies the target(-set) the model index points to and returns
      * the model index of the copy. */
@@ -51,16 +56,19 @@ public Q_SLOTS:
     /** This function deletes the target-set with the same name */
     void deleteTargetSet(const QString &targetSet);
 
+    void moveRowUp(const QModelIndex &index);
+    void moveRowDown(const QModelIndex &index);
+
     const QList<TargetSet> targetSets() const
     {
         return m_targets;
     }
 
     /** Static functions that could be in a different file */
-    static QModelIndex defaultTarget(const QModelIndex &targetSetIndex);
     static const QString command(const QModelIndex &itemIndex);
     static const QString cmdName(const QModelIndex &itemIndex);
     static const QString workDir(const QModelIndex &itemIndex);
+    static const QStringList searchPaths(const QModelIndex &itemIndex);
     static const QString targetName(const QModelIndex &itemIndex);
 
 Q_SIGNALS:
