@@ -7,12 +7,10 @@
 #include "outputstylewidget.h"
 
 #include <KSharedConfig>
-#include <QBrush>
 #include <QCheckBox>
 #include <QFont>
 #include <QFontDatabase>
 #include <QIcon>
-#include <QVariant>
 
 #include <KColorButton>
 #include <KColorScheme>
