@@ -9,10 +9,12 @@
 
 class ConnectionModel;
 class KConfigGroup;
+class QUrl;
 
 #include "connection.h"
 #include <KWallet>
 #include <QSqlQuery>
+#include <QUrl>
 
 class SQLManager : public QObject
 {
@@ -24,22 +26,25 @@ public:
 
     ConnectionModel *connectionModel();
     void createConnection(const Connection &conn);
-    bool testConnection(const Connection &conn, QSqlError &error);
+    static bool testConnection(const Connection &conn, QSqlError &error);
     bool isValidAndOpen(const QString &connection);
 
     KWallet::Wallet *openWallet();
     int storeCredentials(const Connection &conn);
     int readCredentials(const QString &name, QString &password);
+    static const int K_WALLET_CONNECTION_SUCCESSFUL = 0;
+    static const int K_WALLET_CONNECTION_ERROR = -1;
+    static const int K_WALLET_CONNECTION_REJECTED_BY_USER = -2;
 
 public Q_SLOTS:
     void removeConnection(const QString &name);
     void reopenConnection(const QString &name);
-    void loadConnections(KConfigGroup *connectionsGroup);
+    void loadConnections(const KConfigGroup &connectionsGroup);
     void saveConnections(KConfigGroup *connectionsGroup);
     void runQuery(const QString &text, const QString &connection);
 
 protected:
-    void saveConnection(KConfigGroup *connectionsGroup, const Connection &conn);
+    static void saveConnection(KConfigGroup *connectionsGroup, const Connection &conn);
 
 Q_SIGNALS:
     void connectionCreated(const QString &name);
