@@ -7,12 +7,17 @@
 #ifndef LSPCLIENTPLUGIN_H
 #define LSPCLIENTPLUGIN_H
 
-#include <QMap>
 #include <QUrl>
 #include <QVariant>
 
+#include <KTextEditor/Message>
 #include <KTextEditor/Plugin>
 
+#include <map>
+#include <set>
+
+class LSPClientServerManager;
+
 class LSPClientPlugin : public KTextEditor::Plugin
 {
     Q_OBJECT
@@ -59,7 +64,13 @@ public:
     bool m_autoImport = true;
 
     // debug mode?
-    bool m_debugMode = false;
+    const bool m_debugMode;
+
+    // hash of allowed and blacklisted server command lines
+    std::map<QString, bool> m_serverCommandLineToAllowedState;
+
+    // current active dialogs to ask for permission of some command line
+    std::set<QString> m_currentActiveCommandLineDialogs;
 
     // get current config path
     QUrl configPath() const
@@ -67,10 +78,32 @@ public:
         return m_configPath.isEmpty() ? m_defaultConfigPath : m_configPath;
     }
 
-private:
+    /**
+     * Check if given command line is allowed to be executed.
+     * Might ask the user for permission.
+     * @param cmdline full command line including program to check
+     * @return execution allowed?
+     */
+    bool isCommandLineAllowed(const QStringList &cmdline);
+
 Q_SIGNALS:
     // signal settings update
     void update() const;
+
+    void showMessage(KTextEditor::Message::MessageType level, const QString &msg);
+
+private Q_SLOTS:
+    /**
+     * Ask the user via dialog if the given command line shall be allowed.
+     * Will store the result internally and trigger LSP server restart after config change.
+     * Will ensure we just ask once, even if multiple requests queue up.
+     * @param fullCommandLineString full command line string to get permission for
+     */
+    void askForCommandLinePermission(const QString &fullCommandLineString);
+
+private:
+    // server manager to pass along
+    QSharedPointer<LSPClientServerManager> m_serverManager;
 };
 
 #endif
