@@ -10,9 +10,12 @@
 
 #include "lspclientservermanager.h"
 
+#include "hostprocess.h"
+#include "ktexteditor_utils.h"
 #include "lspclient_debug.h"
 
 #include <KLocalizedString>
+#include <KTextEditor/Application>
 #include <KTextEditor/Document>
 #include <KTextEditor/Editor>
 #include <KTextEditor/MainWindow>
@@ -20,9 +23,7 @@
 #include <KTextEditor/View>
 
 #include <QDir>
-#include <QEventLoop>
 #include <QFileInfo>
-#include <QJsonArray>
 #include <QJsonDocument>
 #include <QJsonObject>
 #include <QJsonParseError>
@@ -39,9 +40,14 @@
 typedef QMap<QString, QString> QStringMap;
 Q_DECLARE_METATYPE(QStringMap)
 
-// helper to find a proper root dir for the given document & file name that indicate the root dir
-static QString rootForDocumentAndRootIndicationFileName(KTextEditor::Document *document, const QString &rootIndicationFileName)
+// helper to find a proper root dir for the given document & file name/pattern that indicates the root dir
+static QString findRootForDocument(KTextEditor::Document *document, const QStringList &rootIndicationFileNames, const QStringList &rootIndicationFilePatterns)
 {
+    // skip search if nothing there to look at
+    if (rootIndicationFileNames.isEmpty() && rootIndicationFilePatterns.isEmpty()) {
+        return QString();
+    }
+
     // search only feasible if document is local file
     if (!document->url().isLocalFile()) {
         return QString();
@@ -55,8 +61,18 @@ static QString rootForDocumentAndRootIndicationFileName(KTextEditor::Document *d
         seenDirectories.insert(dir.absolutePath());
 
         // the file that indicates the root dir is there => all fine
-        if (dir.exists(rootIndicationFileName)) {
-            return dir.absolutePath();
+        for (const auto &fileName : rootIndicationFileNames) {
+            if (dir.exists(fileName)) {
+                return dir.absolutePath();
+            }
+        }
+
+        // look for matching file patterns, if any
+        if (!rootIndicationFilePatterns.isEmpty()) {
+            dir.setNameFilters(rootIndicationFilePatterns);
+            if (!dir.entryList().isEmpty()) {
+                return dir.absolutePath();
+            }
         }
 
         // else: cd up, if possible or abort
@@ -69,6 +85,22 @@ static QString rootForDocumentAndRootIndicationFileName(KTextEditor::Document *d
     return QString();
 }
 
+static QStringList indicationDataToStringList(const QJsonValue &indicationData)
+{
+    if (indicationData.isArray()) {
+        QStringList indications;
+        for (auto indication : indicationData.toArray()) {
+            if (indication.isString()) {
+                indications << indication.toString();
+            }
+        }
+
+        return indications;
+    }
+
+    return {};
+};
+
 #include <memory>
 
 // helper guard to handle revision (un)lock
@@ -158,6 +190,8 @@ public:
     }
 };
 
+static const QString PROJECT_PLUGIN{QStringLiteral("kateprojectplugin")};
+
 // helper class to sync document changes to LSP server
 class LSPClientServerManagerImpl : public LSPClientServerManager
 {
@@ -191,7 +225,7 @@ class LSPClientServerManagerImpl : public LSPClientServerManager
     };
 
     LSPClientPlugin *m_plugin;
-    KTextEditor::MainWindow *m_mainWindow;
+    QPointer<QObject> m_projectPlugin;
     // merged default and user config
     QJsonObject m_serverConfig;
     // root -> (mode -> server)
@@ -207,31 +241,31 @@ class LSPClientServerManagerImpl : public LSPClientServerManager
     // most either do not care about the id, or can find out themselves
     // (and might get confused if we pass a not so accurate one)
     QHash<QString, bool> m_documentLanguageId;
-
     typedef QVector<QSharedPointer<LSPClientServer>> ServerList;
 
+    // Servers which were not found to be installed. We use this
+    // variable to avoid warning more than once
+    QSet<QString> m_failedToFindServers;
+
 public:
-    LSPClientServerManagerImpl(LSPClientPlugin *plugin, KTextEditor::MainWindow *mainWin)
+    LSPClientServerManagerImpl(LSPClientPlugin *plugin)
         : m_plugin(plugin)
-        , m_mainWindow(mainWin)
     {
         connect(plugin, &LSPClientPlugin::update, this, &self_type::updateServerConfig);
         QTimer::singleShot(100, this, &self_type::updateServerConfig);
 
         // stay tuned on project situation
-        QObject *projectView = projectPluginView();
-        if (projectView) {
-            // clang-format off
-            connect(projectView,
-                    SIGNAL(pluginProjectAdded(QString,QString)),
-                    this,
-                    SLOT(onProjectAdded(QString,QString)));
-            connect(projectView,
-                    SIGNAL(pluginProjectRemoved(QString,QString)),
-                    this,
-                    SLOT(onProjectRemoved(QString,QString)));
-            // clang-format on
-        }
+        auto app = KTextEditor::Editor::instance()->application();
+        auto h = [this](const QString &name, KTextEditor::Plugin *plugin) {
+            if (name == PROJECT_PLUGIN) {
+                m_projectPlugin = plugin;
+                monitorProjects(plugin);
+            }
+        };
+        connect(app, &KTextEditor::Application::pluginCreated, this, h);
+        auto projectPlugin = app->plugin(PROJECT_PLUGIN);
+        m_projectPlugin = projectPlugin;
+        monitorProjects(projectPlugin);
     }
 
     ~LSPClientServerManagerImpl() override
@@ -320,9 +354,9 @@ public:
         return QString();
     }
 
-    QObject *projectPluginView()
+    QObject *projectPluginView(KTextEditor::MainWindow *mainWindow)
     {
-        return m_mainWindow->pluginView(QStringLiteral("kateprojectplugin"));
+        return mainWindow->pluginView(PROJECT_PLUGIN);
     }
 
     QString documentLanguageId(const QString mode)
@@ -420,7 +454,7 @@ private:
     void showMessage(const QString &msg, KTextEditor::Message::MessageType level)
     {
         // inform interested view(er) which will decide how/where to show
-        Q_EMIT LSPClientServerManager::showMessage(level, msg);
+        Q_EMIT m_plugin->showMessage(level, msg);
     }
 
     // caller ensures that servers are no longer present in m_servers
@@ -558,10 +592,9 @@ private:
             return nullptr;
         }
 
-        QObject *projectView = projectPluginView();
-        // preserve raw QString value so it can be used and tested that way below
-        const auto projectBase = projectView ? projectView->property("projectBaseDir").toString() : QString();
-        const auto &projectMap = projectView ? projectView->property("projectMap").toMap() : QVariantMap();
+        // get project plugin infos if available
+        const auto projectBase = Utils::projectBaseDirForDocument(document);
+        const auto projectMap = Utils::projectMapForDocument(document);
 
         // merge with project specific
         auto projectConfig = QJsonDocument::fromVariant(projectMap).object().value(QStringLiteral("lspclient")).toObject();
@@ -622,20 +655,11 @@ private:
          * clangd does
          */
         if (!rootpath) {
-            const auto fileNamesForDetection = serverConfig.value(QStringLiteral("rootIndicationFileNames"));
-            if (fileNamesForDetection.isArray()) {
-                // we try each file name alternative in the listed order
-                // this allows to have preferences
-                const auto fileNames = fileNamesForDetection.toArray();
-                for (auto name : fileNames) {
-                    if (name.isString()) {
-                        auto root = rootForDocumentAndRootIndicationFileName(document, name.toString());
-                        if (!root.isEmpty()) {
-                            rootpath = root;
-                            break;
-                        }
-                    }
-                }
+            const auto fileNamesForDetection = indicationDataToStringList(serverConfig.value(QStringLiteral("rootIndicationFileNames")));
+            const auto filePatternsForDetection = indicationDataToStringList(serverConfig.value(QStringLiteral("rootIndicationFilePatterns")));
+            const auto root = findRootForDocument(document, fileNamesForDetection, filePatternsForDetection);
+            if (!root.isEmpty()) {
+                rootpath = root;
             }
         }
 
@@ -686,9 +710,9 @@ private:
             }
         }
 
+        QStringList cmdline;
         if (!server) {
-            QStringList cmdline;
-
+            // need to find command line for server
             // choose debug command line for debug mode, fallback to command
             auto vcmdline = serverConfig.value(m_plugin->m_debugMode ? QStringLiteral("commandDebug") : QStringLiteral("command"));
             if (vcmdline.isUndefined()) {
@@ -710,59 +734,41 @@ private:
             for (auto &e : cmdline) {
                 editor->expandText(e, view, e);
             }
+        }
 
-            if (cmdline.length() > 0) {
-                // ensure we always only take the server executable from the PATH or user defined paths
-                // QProcess will take the executable even just from current working directory without this => BAD
-                auto cmd = QStandardPaths::findExecutable(cmdline[0]);
+        if (cmdline.length() > 0) {
+            // always update some info
+            // (even if eventually no server found/started)
+            serverinfo.settings = serverConfig.value(QStringLiteral("settings"));
+            serverinfo.started = QTime::currentTime();
+            serverinfo.url = serverConfig.value(QStringLiteral("url")).toString();
+            // leave failcount as-is
+            serverinfo.useWorkspace = useWorkspace;
 
-                // optionally search in supplied path(s)
-                const auto vpath = serverConfig.value(QStringLiteral("path")).toArray();
-                if (cmd.isEmpty() && !vpath.isEmpty()) {
-                    // collect and expand in case home dir or other (environment) variable reference is used
-                    QStringList path;
-                    for (const auto &e : vpath) {
-                        auto p = e.toString();
-                        editor->expandText(p, view, p);
-                        path.push_back(p);
-                    }
-                    cmd = QStandardPaths::findExecutable(cmdline[0], path);
+            // ensure we always only take the server executable from the PATH or user defined paths
+            // QProcess will take the executable even just from current working directory without this => BAD
+            auto cmd = safeExecutableName(cmdline[0]);
+
+            // optionally search in supplied path(s)
+            const auto vpath = serverConfig.value(QStringLiteral("path")).toArray();
+            if (cmd.isEmpty() && !vpath.isEmpty()) {
+                // collect and expand in case home dir or other (environment) variable reference is used
+                QStringList path;
+                for (const auto &e : vpath) {
+                    auto p = e.toString();
+                    editor->expandText(p, view, p);
+                    path.push_back(p);
                 }
+                cmd = safeExecutableName(cmdline[0], path);
+            }
 
-                // we can only start the stuff if we did find the binary in the paths
-                if (!cmd.isEmpty()) {
-                    // use full path to avoid security issues
-                    cmdline[0] = cmd;
-
-                    // an empty list is always passed here (or null)
-                    // the initial list is provided/updated using notification after start
-                    // since that is what a server is more aware of
-                    // and should support if it declares workspace folder capable
-                    // (as opposed to the new initialization property)
-                    LSPClientServer::FoldersType folders;
-                    if (useWorkspace) {
-                        folders = QList<LSPWorkspaceFolder>();
-                    }
-                    server.reset(new LSPClientServer(cmdline, root, realLangId, serverConfig.value(QStringLiteral("initializationOptions")), folders));
-                    connect(server.data(), &LSPClientServer::stateChanged, this, &self_type::onStateChanged, Qt::UniqueConnection);
-                    if (!server->start()) {
-                        QString message = i18n("Failed to start server: %1", cmdline.join(QLatin1Char(' ')));
-                        const auto url = serverConfig.value(QStringLiteral("url")).toString();
-                        if (!url.isEmpty()) {
-                            message += QStringLiteral("\n") + i18n("Please check your PATH for the binary");
-                            message += QStringLiteral("\n") + i18n("See also %1 for installation or details", url);
-                        }
-                        showMessage(message, KTextEditor::Message::Warning);
-                    } else {
-                        showMessage(i18n("Started server %2: %1", cmdline.join(QLatin1Char(' ')), serverDescription(server.data())),
-                                    KTextEditor::Message::Positive);
-                        using namespace std::placeholders;
-                        server->connect(server.data(), &LSPClientServer::logMessage, this, std::bind(&self_type::onMessage, this, true, _1));
-                        server->connect(server.data(), &LSPClientServer::showMessage, this, std::bind(&self_type::onMessage, this, false, _1));
-                        server->connect(server.data(), &LSPClientServer::workDoneProgress, this, &self_type::onWorkDoneProgress);
-                        server->connect(server.data(), &LSPClientServer::workspaceFolders, this, &self_type::onWorkspaceFolders, Qt::UniqueConnection);
-                    }
-                } else {
+            // we can only start the stuff if we did find the binary in the paths
+            if (!cmd.isEmpty()) {
+                // use full path to avoid security issues
+                cmdline[0] = cmd;
+            } else {
+                if (!m_failedToFindServers.contains(cmdline[0])) {
+                    m_failedToFindServers.insert(cmdline[0]);
                     // we didn't find the server binary at all!
                     QString message = i18n("Failed to find server binary: %1", cmdline[0]);
                     const auto url = serverConfig.value(QStringLiteral("url")).toString();
@@ -772,14 +778,47 @@ private:
                     }
                     showMessage(message, KTextEditor::Message::Warning);
                 }
-
-                serverinfo.settings = serverConfig.value(QStringLiteral("settings"));
-                serverinfo.started = QTime::currentTime();
-                serverinfo.url = serverConfig.value(QStringLiteral("url")).toString();
-                // leave failcount as-is
-                serverinfo.useWorkspace = useWorkspace;
+                // clear to cut branch below
+                cmdline.clear();
             }
         }
+
+        // check if allowed to start, function will query user if needed and emit messages
+        if (cmdline.length() > 0 && !m_plugin->isCommandLineAllowed(cmdline)) {
+            cmdline.clear();
+        }
+
+        // made it here with a command line; spin up server
+        if (cmdline.length() > 0) {
+            // an empty list is always passed here (or null)
+            // the initial list is provided/updated using notification after start
+            // since that is what a server is more aware of
+            // and should support if it declares workspace folder capable
+            // (as opposed to the new initialization property)
+            LSPClientServer::FoldersType folders;
+            if (useWorkspace) {
+                folders = QList<LSPWorkspaceFolder>();
+            }
+            server.reset(new LSPClientServer(cmdline, root, realLangId, serverConfig.value(QStringLiteral("initializationOptions")), folders));
+            connect(server.data(), &LSPClientServer::stateChanged, this, &self_type::onStateChanged, Qt::UniqueConnection);
+            if (!server->start(m_plugin->m_debugMode)) {
+                QString message = i18n("Failed to start server: %1", cmdline.join(QLatin1Char(' ')));
+                const auto url = serverConfig.value(QStringLiteral("url")).toString();
+                if (!url.isEmpty()) {
+                    message += QStringLiteral("\n") + i18n("Please check your PATH for the binary");
+                    message += QStringLiteral("\n") + i18n("See also %1 for installation or details", url);
+                }
+                showMessage(message, KTextEditor::Message::Warning);
+            } else {
+                showMessage(i18n("Started server %2: %1", cmdline.join(QLatin1Char(' ')), serverDescription(server.data())), KTextEditor::Message::Positive);
+                using namespace std::placeholders;
+                server->connect(server.data(), &LSPClientServer::logMessage, this, std::bind(&self_type::onMessage, this, true, _1));
+                server->connect(server.data(), &LSPClientServer::showMessage, this, std::bind(&self_type::onMessage, this, false, _1));
+                server->connect(server.data(), &LSPClientServer::workDoneProgress, this, &self_type::onWorkDoneProgress);
+                server->connect(server.data(), &LSPClientServer::workspaceFolders, this, &self_type::onWorkspaceFolders, Qt::UniqueConnection);
+            }
+        }
+        // set out param value
         mergedConfig = serverConfig;
         return (server && server->state() == LSPClientServer::State::Running) ? server : nullptr;
     }
@@ -834,6 +873,7 @@ private:
                 m_documentLanguageId[it.key()] = docLanguageId.toBool();
             }
         }
+        m_failedToFindServers.clear();
 
         // we could (but do not) perform restartAll here;
         // for now let's leave that up to user
@@ -1029,14 +1069,19 @@ private:
         Q_EMIT serverWorkDoneProgress(server, params);
     }
 
+    static std::pair<QString, QString> getProjectNameDir(const QObject *kateProject)
+    {
+        return {kateProject->property("name").toString(), kateProject->property("baseDir").toString()};
+    }
+
     QList<LSPWorkspaceFolder> currentWorkspaceFolders()
     {
         QList<LSPWorkspaceFolder> folders;
-        auto projectView = projectPluginView();
-        if (projectView) {
-            auto projectsMap = projectView->property("allProjects").value<QStringMap>();
-            for (auto it = projectsMap.begin(); it != projectsMap.end(); ++it) {
-                folders.push_back(workspaceFolder(it.key(), it.value()));
+        if (m_projectPlugin) {
+            auto projects = m_projectPlugin->property("projects").value<QObjectList>();
+            for (auto proj : projects) {
+                auto props = getProjectNameDir(proj);
+                folders.push_back(workspaceFolder(props.second, props.first));
             }
         }
         return folders;
@@ -1047,8 +1092,11 @@ private:
         return {QUrl::fromLocalFile(baseDir), name};
     }
 
-    void updateWorkspace(bool added, const QString &baseDir, const QString &name)
+    void updateWorkspace(bool added, const QObject *project)
     {
+        auto props = getProjectNameDir(project);
+        auto &name = props.first;
+        auto &baseDir = props.second;
         qCInfo(LSPCLIENT) << "update workspace" << added << baseDir << name;
         for (const auto &u : qAsConst(m_servers)) {
             for (const auto &si : u) {
@@ -1064,14 +1112,32 @@ private:
         }
     }
 
-    Q_SLOT void onProjectAdded(QString baseDir, QString name)
+    Q_SLOT void onProjectAdded(QObject *project)
     {
-        updateWorkspace(true, baseDir, name);
+        updateWorkspace(true, project);
     }
 
-    Q_SLOT void onProjectRemoved(QString baseDir, QString name)
+    Q_SLOT void onProjectRemoved(QObject *project)
     {
-        updateWorkspace(false, baseDir, name);
+        updateWorkspace(false, project);
+    }
+
+    void monitorProjects(KTextEditor::Plugin *projectPlugin)
+    {
+        if (projectPlugin) {
+            // clang-format off
+            auto c = connect(projectPlugin,
+                        SIGNAL(projectAdded(QObject*)),
+                        this,
+                        SLOT(onProjectAdded(QObject*)),
+                        Qt::UniqueConnection);
+            c = connect(projectPlugin,
+                        SIGNAL(projectRemoved(QObject*)),
+                        this,
+                        SLOT(onProjectRemoved(QObject*)),
+                        Qt::UniqueConnection);
+            // clang-format on
+        }
     }
 
     void onWorkspaceFolders(const WorkspaceFoldersReplyHandler &h, bool &handled)
@@ -1087,9 +1153,9 @@ private:
     }
 };
 
-QSharedPointer<LSPClientServerManager> LSPClientServerManager::new_(LSPClientPlugin *plugin, KTextEditor::MainWindow *mainWin)
+QSharedPointer<LSPClientServerManager> LSPClientServerManager::new_(LSPClientPlugin *plugin)
 {
-    return QSharedPointer<LSPClientServerManager>(new LSPClientServerManagerImpl(plugin, mainWin));
+    return QSharedPointer<LSPClientServerManager>(new LSPClientServerManagerImpl(plugin));
 }
 
 #include "lspclientservermanager.moc"
