@@ -7,6 +7,7 @@
 
 #include "lspclientsymbolview.h"
 
+#include <KFuzzyMatcher>
 #include <KLineEdit>
 #include <KLocalizedString>
 #include <QSortFilterProxyModel>
@@ -27,8 +28,6 @@
 #include <memory>
 #include <utility>
 
-#include <kfts_fuzzy_match.h>
-
 // TODO: Make this globally available in shared/
 enum SymbolViewRoles { SymbolRange = Qt::UserRole, ScoreRole, IsPlaceholder };
 
@@ -148,12 +147,11 @@ protected:
             return true;
         }
 
-        int score = 0;
         const auto idx = sourceModel()->index(sourceRow, 0, sourceParent);
         const QString symbol = idx.data().toString();
-        const bool res = kfts::fuzzy_match(m_pattern, symbol, score);
-        sourceModel()->setData(idx, score, SymbolViewRoles::ScoreRole);
-        return res;
+        const auto res = KFuzzyMatcher::match(m_pattern, symbol);
+        sourceModel()->setData(idx, res.score, SymbolViewRoles::ScoreRole);
+        return res.matched;
     }
 
 private:
@@ -236,7 +234,7 @@ public:
         m_toolview.reset(m_mainWindow->createToolView(plugin,
                                                       QStringLiteral("lspclient_symbol_outline"),
                                                       KTextEditor::MainWindow::Left,
-                                                      QIcon::fromTheme(QStringLiteral("code-context")),
+                                                      QIcon::fromTheme(QStringLiteral("quickopen-class")),
                                                       i18n("Symbol Outline")));
 
         m_symbols = new QTreeView(m_toolview.data());
