@@ -1,5 +1,4 @@
 #include <QDialog>
-#include <QLabel>
 #include <QStandardItemModel>
 #include <QTreeView>
 
@@ -13,6 +12,7 @@ public:
 private:
     void loadBranches(const QString &dotGitPath);
     void updateLabel(QStandardItem *item);
+    void onCheckAllClicked(bool);
     QStandardItemModel m_model;
     QTreeView m_listView;
 };
