@@ -5,8 +5,10 @@
 */
 
 #include "comparebranchesview.h"
+#include "diffparams.h"
 #include "kateprojectpluginview.h"
 #include "kateprojectworker.h"
+#include "ktexteditor_utils.h"
 
 #include <gitprocess.h>
 
@@ -161,12 +163,17 @@ void CompareBranchesView::showDiff(const QModelIndex &idx)
     if (!setupGitProcess(git, m_gitDir, {QStringLiteral("diff"), QStringLiteral("%1...%2").arg(m_fromBr).arg(m_toBr), QStringLiteral("--"), file})) {
         return;
     }
-    git.start(QProcess::ReadOnly);
+    startHostProcess(git, QProcess::ReadOnly);
 
     if (git.waitForStarted() && git.waitForFinished(-1)) {
         if (git.exitStatus() != QProcess::NormalExit || git.exitCode() != 0) {
             return;
         }
     }
-    m_pluginView->showDiffInFixedView(git.readAllStandardOutput());
+
+    DiffParams d;
+    d.tabTitle = QStringLiteral("Diff %1[%2 .. %3]").arg(Utils::fileNameFromPath(file)).arg(m_fromBr).arg(m_toBr);
+    d.workingDir = m_gitDir;
+    d.arguments = git.arguments();
+    Utils::showDiff(git.readAllStandardOutput(), d, m_pluginView->mainWindow());
 }
