@@ -45,8 +45,12 @@ public:
     explicit GitWidget(KateProject *project, KTextEditor::MainWindow *mainWindow = nullptr, KateProjectPluginView *pluginView = nullptr);
     ~GitWidget() override;
 
+    void init();
+
     bool eventFilter(QObject *o, QEvent *e) override;
 
+    void showEvent(QShowEvent *e) override;
+
     /**
      * Trigger the GitWidget to update itself.
      * It is safe to call it repeatedly in a short time, due to delayed update after the last call.
@@ -60,10 +64,29 @@ public:
 
     QString dotGitPath() const
     {
-        return m_gitPath;
+        return m_activeGitDirPath;
+    }
+
+    QString indexPath() const
+    {
+        if (m_activeGitDirPath == m_topLevelGitPath) {
+            return m_activeGitDirPath + QStringLiteral(".git/index");
+        }
+        // Should we support submodules?
+        return QString();
     }
 
 private:
+    /** These ends with "/", always remember this */
+
+    // This variable contains the current active git path
+    // being tracked by this widget
+    QString m_activeGitDirPath;
+    // This variable contains the topLevel git path
+    QString m_topLevelGitPath;
+    // This variable contains all submodule paths
+    QStringList m_submodulePaths;
+
     /**
      * Helper to avoid multiple reloads at a time
      * @see slotUpdateStatus
@@ -79,13 +102,12 @@ private:
     GitWidgetTreeView *m_treeView;
     GitStatusModel *m_model;
     QLineEdit *m_filterLineEdit;
-    /** This ends with "/", always remember this */
-    QString m_gitPath;
     QFutureWatcher<GitUtils::GitParsedStatus> m_gitStatusWatcher;
     QString m_commitMessage;
     KTextEditor::MainWindow *m_mainWin;
     QMenu *m_gitMenu;
     KateProjectPluginView *m_pluginView;
+    bool m_initialized = false;
 
     QWidget *m_mainView;
     QStackedWidget *m_stackWidget;
@@ -93,10 +115,13 @@ private:
     using CancelHandle = QPointer<QProcess>;
     CancelHandle m_cancelHandle;
 
+    void setDotGitPath();
+    void setSubmodulesPaths();
+    void setActiveGitDir();
+
     QProcess *gitp(const QStringList &arguments);
 
     void buildMenu(KActionCollection *ac);
-    void setDotGitPath();
     void runGitCmd(const QStringList &args, const QString &i18error);
     void runPushPullCmd(const QStringList &args);
     void stage(const QStringList &files, bool = false);
@@ -107,7 +132,6 @@ private:
     void showDiff(const QString &file, bool staged);
     void launchExternalDiffTool(const QString &file, bool staged);
     void commitChanges(const QString &msg, const QString &desc, bool signOff, bool amend = false);
-    void applyDiff(const QString &fileName, bool staged, bool hunk, KTextEditor::View *v);
     void branchCompareFiles(const QString &from, const QString &to);
 
     QMenu *stashMenu(KActionCollection *pCollection);
@@ -116,7 +140,6 @@ private:
     void treeViewContextMenuEvent(QContextMenuEvent *e);
     void selectedContextMenu(QContextMenuEvent *e);
 
-    QString getDiff(KTextEditor::View *view, bool hunk, bool alreadyStaged);
     void createStashDialog(StashMode m, const QString &gitPath);
 
     void enableCancel(QProcess *git);
