@@ -8,10 +8,15 @@
 #ifndef KATE_PROJECT_CODE_ANALYSIS_TOOL_H
 #define KATE_PROJECT_CODE_ANALYSIS_TOOL_H
 
+#include <QObject>
 #include <QString>
 #include <QStringList>
 
-#include "kateproject.h"
+class KateProject;
+namespace KTextEditor
+{
+class MainWindow;
+}
 
 /**
  * Information provider for a code analysis tool
@@ -27,6 +32,8 @@ protected:
      */
     KateProject *m_project = nullptr;
 
+    KTextEditor::MainWindow *m_mainWindow;
+
 public:
     ~KateProjectCodeAnalysisTool() override;
 
@@ -116,6 +123,8 @@ public:
      */
     void setActualFilesCount(int count);
 
+    void setMainWindow(KTextEditor::MainWindow *mainWin);
+
 private:
     int m_filesCount = 0;
 };
