@@ -9,10 +9,9 @@
 #ifndef KATEPROJECTFILTERMODEL_H
 #define KATEPROJECTFILTERMODEL_H
 
-#include <QDebug>
 #include <QSortFilterProxyModel>
 
-#include <kfts_fuzzy_match.h>
+#include <KFuzzyMatcher>
 
 class KateProjectFilterProxyModel : public QSortFilterProxyModel
 {
@@ -45,7 +44,7 @@ protected:
         }
 
         const QString file = index.data().toString();
-        return kfts::fuzzy_match_simple(m_pattern, file);
+        return KFuzzyMatcher::matchSimple(m_pattern, file);
     }
 
 private:
