@@ -15,8 +15,6 @@
 #include <QAction>
 #include <QVBoxLayout>
 
-#include <KTextEditor/MainWindow>
-
 KateProjectInfoViewIndex::KateProjectInfoViewIndex(KateProjectPluginView *pluginView, KateProject *project, QWidget *parent)
     : QWidget(parent)
     , m_pluginView(pluginView)
@@ -49,6 +47,7 @@ KateProjectInfoViewIndex::KateProjectInfoViewIndex(KateProjectPluginView *plugin
      */
     QVBoxLayout *layout = new QVBoxLayout;
     layout->setSpacing(0);
+    layout->setContentsMargins(0, 0, 0, 0);
     layout->addWidget(m_lineEdit);
     layout->addWidget(m_treeView);
     setLayout(layout);
