@@ -9,9 +9,7 @@
 #include "kateproject.h"
 
 #include <QCoreApplication>
-#include <QDir>
 #include <QFile>
-#include <QFileInfo>
 #include <QIcon>
 #include <QMessageBox>
 #include <QMimeDatabase>
@@ -26,6 +24,11 @@ KateProjectItem::KateProjectItem(Type type, const QString &text)
     : QStandardItem(text)
     , m_type(type)
 {
+    if (type == File) {
+        auto flags = this->flags();
+        flags.setFlag(Qt::ItemIsDropEnabled, false);
+        setFlags(flags);
+    }
 }
 
 KateProjectItem::~KateProjectItem()
