@@ -10,6 +10,7 @@
 #include "kateproject.h"
 #include "kateprojectconfigpage.h"
 #include "kateprojectpluginview.h"
+#include "ktexteditor_utils.h"
 
 #include <kcoreaddons_version.h>
 #include <ktexteditor/application.h>
@@ -77,44 +78,7 @@ KateProjectPlugin::KateProjectPlugin(QObject *parent, const QList<QVariant> &)
     // make project plugin variables known to KTextEditor::Editor
     registerVariables();
 
-    // open directories as projects
-    KateProject *projectToActivate = nullptr;
-    auto args = qApp->arguments();
-    args.removeFirst(); // The first argument is the executable name
-    for (const QString &arg : qAsConst(args)) {
-        QFileInfo info(arg);
-        if (info.isDir()) {
-            projectToActivate = projectForDir(info.absoluteFilePath(), true);
-        }
-    }
-
-#ifdef HAVE_CTERMID
-    /**
-     * open project for our current working directory, if this kate has a terminal
-     * https://stackoverflow.com/questions/1312922/detect-if-stdin-is-a-terminal-or-pipe-in-c-c-qt
-     */
-    if (!projectToActivate) {
-        char tty[L_ctermid + 1] = {0};
-        ctermid(tty);
-        if (int fd = ::open(tty, O_RDONLY); fd >= 0) {
-            projectToActivate = projectForDir(QDir::current());
-            ::close(fd);
-        }
-    }
-#endif
-
-    /**
-     * delay activation after session restore
-     */
-    if (projectToActivate) {
-        QTimer::singleShot(0, projectToActivate, [this, projectToActivate]() {
-            Q_EMIT activateProject(projectToActivate);
-        });
-    }
-
-    /**
-     * forward to meta-object system friendly version
-     */
+    // forward to meta-object system friendly version
     connect(this, &KateProjectPlugin::projectCreated, this, &KateProjectPlugin::projectAdded);
     connect(this, &KateProjectPlugin::pluginViewProjectClosing, this, &KateProjectPlugin::projectRemoved);
 }
@@ -552,7 +516,7 @@ void KateProjectPlugin::readConfig()
     m_multiProjectGoto = config.readEntry("multiProjectCompletion", false);
 
     m_gitNumStat = config.readEntry("gitStatusNumStat", true);
-    m_singleClickAction = (ClickAction)config.readEntry("gitStatusSingleClick", (int)ClickAction::ShowDiff);
+    m_singleClickAction = (ClickAction)config.readEntry("gitStatusSingleClick", (int)ClickAction::NoAction);
     m_doubleClickAction = (ClickAction)config.readEntry("gitStatusDoubleClick", (int)ClickAction::StageUnstage);
 
     m_restoreProjectsForSession = config.readEntry("restoreProjectsForSessions", false);
@@ -671,6 +635,49 @@ void KateProjectPlugin::readSessionConfig(const KConfigGroup &config)
             // we might arrive here if invalid data is store, just ignore that, we just loose session data
         }
     }
+
+    // always load projects from command line or current working directory first time we arrive here
+    if (m_initialReadSessionConfigDone) {
+        return;
+    }
+    m_initialReadSessionConfigDone = true;
+
+    /**
+     * delayed activation after session restore
+     * we do this both for session restoration to not take preference and
+     * to be able to signal errors during project loading via message() signals
+     */
+    KateProject *projectToActivate = nullptr;
+
+    // open directories as projects
+    auto args = qApp->arguments();
+    args.removeFirst(); // The first argument is the executable name
+    for (const QString &arg : qAsConst(args)) {
+        QFileInfo info(arg);
+        if (info.isDir()) {
+            projectToActivate = projectForDir(info.absoluteFilePath(), true);
+        }
+    }
+
+#ifdef HAVE_CTERMID
+    /**
+     * open project for our current working directory, if this kate has a terminal
+     * https://stackoverflow.com/questions/1312922/detect-if-stdin-is-a-terminal-or-pipe-in-c-c-qt
+     */
+    if (!projectToActivate) {
+        char tty[L_ctermid + 1] = {0};
+        ctermid(tty);
+        if (int fd = ::open(tty, O_RDONLY); fd >= 0) {
+            projectToActivate = projectForDir(QDir::current());
+            ::close(fd);
+        }
+    }
+#endif
+
+    // if we have some project opened, ensure it is the active one, this happens after session restore
+    if (projectToActivate) {
+        Q_EMIT activateProject(projectToActivate);
+    }
 }
 
 void KateProjectPlugin::writeSessionConfig(KConfigGroup &config)
@@ -698,3 +705,33 @@ void KateProjectPlugin::writeSessionConfig(KConfigGroup &config)
     }
     config.writeEntry("projects", projectList);
 }
+
+void KateProjectPlugin::sendMessage(const QString &text, bool error)
+{
+    QVariantMap genericMessage;
+    genericMessage.insert(QStringLiteral("type"), error ? QStringLiteral("Error") : QStringLiteral("Info"));
+    genericMessage.insert(QStringLiteral("category"), i18n("Project"));
+    genericMessage.insert(QStringLiteral("categoryIcon"), QIcon::fromTheme(QStringLiteral("project-open")));
+    genericMessage.insert(QStringLiteral("text"), text);
+    Utils::showMessage(genericMessage);
+}
+
+QString KateProjectPlugin::projectBaseDirForDocument(KTextEditor::Document *doc)
+{
+    // quick lookup first, then search
+    auto project = projectForDocument(doc);
+    if (!project) {
+        project = projectForUrl(doc->url());
+    }
+    return project ? project->baseDir() : QString();
+}
+
+QVariantMap KateProjectPlugin::projectMapForDocument(KTextEditor::Document *doc)
+{
+    // quick lookup first, then search
+    auto project = projectForDocument(doc);
+    if (!project) {
+        project = projectForUrl(doc->url());
+    }
+    return project ? project->projectMap() : QVariantMap();
+}
