@@ -6,6 +6,7 @@
  */
 
 #include "kateprojecttreeviewcontextmenu.h"
+#include "filehistorywidget.h"
 #include "git/gitutils.h"
 #include "kateproject.h"
 #include "kateprojectinfoviewterminal.h"
@@ -13,11 +14,16 @@
 
 #include <KApplicationTrader>
 #include <KIO/ApplicationLauncherJob>
+#include <kio_version.h>
+#if KIO_VERSION >= QT_VERSION_CHECK(5, 98, 0)
+#include <KIO/JobUiDelegateFactory>
+#else
 #include <KIO/JobUiDelegate>
+#endif
 #include <KIO/OpenFileManagerWindowJob>
 #include <KLocalizedString>
-#include <KNS3/KMoreTools>
-#include <KNS3/KMoreToolsMenuFactory>
+#include <KMoreTools>
+#include <KMoreToolsMenuFactory>
 #include <KPropertiesDialog>
 #include <KTerminalLauncherJob>
 
@@ -34,14 +40,12 @@
 #include <QMimeType>
 #include <QStandardPaths>
 
-#include <ktexteditor/editor.h>
 #include <ktexteditor/application.h>
+#include <ktexteditor/editor.h>
 
-
-
-static QString getName()
+static QString getName(QWidget *parent)
 {
-    QInputDialog dlg;
+    QInputDialog dlg(parent);
     dlg.setLabelText(i18n("Enter name:"));
     dlg.setOkButtonText(i18n("Add"));
     dlg.setInputMode(QInputDialog::TextInput);
@@ -59,12 +63,12 @@ void KateProjectTreeViewContextMenu::exec(const QString &filename, const QModelI
     /**
      * Create context menu
      */
-    QMenu menu;
+    QMenu menu(parent);
 
     /**
      * Copy Path, always available, put that to the top
      */
-    QAction *copyAction = menu.addAction(QIcon::fromTheme(QStringLiteral("edit-copy")), i18n("Copy File Path"));
+    QAction *copyAction = menu.addAction(QIcon::fromTheme(QStringLiteral("edit-copy-path")), i18n("Copy Location"));
 
     QAction *addFile = nullptr;
     QAction *addFolder = nullptr;
@@ -92,6 +96,7 @@ void KateProjectTreeViewContextMenu::exec(const QString &filename, const QModelI
      */
     menu.addSeparator();
     QMenu *openWithMenu = menu.addMenu(i18n("Open With"));
+    openWithMenu->setIcon(QIcon::fromTheme(QStringLiteral("system-run")));
     QMimeType mimeType = QMimeDatabase().mimeTypeForFile(filename);
     const KService::List offers = KApplicationTrader::queryByMimeType(mimeType.name());
     // For each one, insert a menu item...
@@ -146,23 +151,24 @@ void KateProjectTreeViewContextMenu::exec(const QString &filename, const QModelI
         // If app is null, ApplicationLauncherJob will invoke the open-with dialog
         auto *job = new KIO::ApplicationLauncherJob(app);
         job->setUrls({QUrl::fromLocalFile(filename)});
+#if KIO_VERSION >= QT_VERSION_CHECK(5, 98, 0)
+        job->setUiDelegate(KIO::createDefaultJobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, parent));
+#else
         job->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, parent));
+#endif
         job->start();
     };
-    
-    auto handleDeleteFile = [parent, index](const QString &path)
-    {
-        //message box
+
+    auto handleDeleteFile = [parent, index](const QString &path) {
+        // message box
         const QString title = i18n("Delete File");
         const QString text = i18n("Do you want to delete the file '%1'?", path);
         if (QMessageBox::Yes == QMessageBox::question(parent, title, text, QMessageBox::No | QMessageBox::Yes, QMessageBox::Yes)) {
-            const QList< KTextEditor::Document* > openDocuments = KTextEditor::Editor::instance()->application()->documents();
-              
-            //if is open, close
-            for(auto doc : openDocuments)
-            {
-                if(doc->url().adjusted(QUrl::RemoveScheme) == QUrl(path).adjusted(QUrl::RemoveScheme))
-                {
+            const QList<KTextEditor::Document *> openDocuments = KTextEditor::Editor::instance()->application()->documents();
+
+            // if is open, close
+            for (auto doc : openDocuments) {
+                if (doc->url().adjusted(QUrl::RemoveScheme) == QUrl(path).adjusted(QUrl::RemoveScheme)) {
                     KTextEditor::Editor::instance()->application()->closeDocument(doc);
                     break;
                 }
@@ -197,7 +203,7 @@ void KateProjectTreeViewContextMenu::exec(const QString &filename, const QModelI
         } else if (action == filePropertiesAction) {
             // code copied and adapted from frameworks/kio/src/filewidgets/knewfilemenu.cpp
             KFileItem fileItem(QUrl::fromLocalFile(filename));
-            QDialog *dlg = new KPropertiesDialog(fileItem);
+            QDialog *dlg = new KPropertiesDialog(fileItem, parent);
             dlg->setAttribute(Qt::WA_DeleteOnClose);
             dlg->show();
         } else if (rename && action == rename) {
@@ -215,14 +221,14 @@ void KateProjectTreeViewContextMenu::exec(const QString &filename, const QModelI
             /** start the edit */
             parent->edit(index);
         } else if (action == fileHistory) {
-            showFileHistory(index.data(Qt::UserRole).toString());
+            FileHistory::showFileHistory(index.data(Qt::UserRole).toString());
         } else if (addFile && action == addFile) {
-            QString name = getName();
+            QString name = getName(parent);
             if (!name.isEmpty()) {
                 parent->addFile(index, name);
             }
         } else if (addFolder && action == addFolder) {
-            QString name = getName();
+            QString name = getName(parent);
             if (!name.isEmpty()) {
                 parent->addDirectory(index, name);
             }
