@@ -8,6 +8,7 @@
 #include "kateprojectworker.h"
 #include "kateprojectitem.h"
 
+#include <bytearraysplitter.h>
 #include <gitprocess.h>
 
 #include "hostprocess.h"
@@ -18,10 +19,8 @@
 #include <QProcess>
 #include <QRegularExpression>
 #include <QSet>
-#include <QSettings>
 #include <QStandardPaths>
 #include <QThread>
-#include <QTime>
 #include <QtConcurrent>
 
 #include <algorithm>
@@ -290,7 +289,14 @@ void KateProjectWorker::loadFilesEntry(QStandardItem *parent,
     /**
      * get list of files for this directory, might query the VCS
      */
-    QVector<QString> files = findFiles(dir, filesEntry);
+    const QVector<QString> files = findFiles(dir, filesEntry);
+
+    QStringList excludeFolderPatterns = m_projectMap.value(QStringLiteral("exclude_patterns")).toStringList();
+    std::vector<QRegularExpression> excludeRegexps;
+    excludeRegexps.reserve(excludeFolderPatterns.size());
+    for (const auto &pattern : excludeFolderPatterns) {
+        excludeRegexps.push_back(QRegularExpression(pattern, QRegularExpression::DontCaptureOption));
+    }
 
     /**
      * sort out non-files
@@ -304,7 +310,7 @@ void KateProjectWorker::loadFilesEntry(QStandardItem *parent,
     preparedItems.reserve(files.size());
     for (const auto &item : files)
         preparedItems.emplace_back(item, QString(), nullptr);
-    QtConcurrent::blockingMap(preparedItems, [dirPath](std::tuple<QString, QString, KateProjectItem *> &item) {
+    QtConcurrent::blockingMap(preparedItems, [excludeFolderPatterns, dirPath, excludeRegexps](std::tuple<QString, QString, KateProjectItem *> &item) {
         /**
          * cheap file name computation
          * we do this A LOT, QFileInfo is very expensive just for this operation
@@ -312,6 +318,13 @@ void KateProjectWorker::loadFilesEntry(QStandardItem *parent,
          */
         auto &[filePath, fullFilePath, projectItem] = item;
         fullFilePath = dirPath + filePath;
+
+        for (const auto &excludePattern : excludeRegexps) {
+            if (excludePattern.match(filePath).hasMatch()) {
+                return;
+            }
+        }
+
         const int slashIndex = filePath.lastIndexOf(QLatin1Char('/'));
         const QString fileName = (slashIndex < 0) ? filePath : filePath.mid(slashIndex + 1);
         filePath = (slashIndex < 0) ? QString() : filePath.left(slashIndex);
@@ -468,16 +481,15 @@ QVector<QString> KateProjectWorker::gitFiles(const QDir &dir, bool recursive, co
         return files;
     }
 
-    const QList<QByteArray> byteArrayList = git.readAllStandardOutput().split('\0');
-    files.reserve(byteArrayList.size());
-    for (const QByteArray &byteArray : byteArrayList) {
-        if (byteArray.isEmpty()) {
+    const QByteArray b = git.readAllStandardOutput();
+    for (strview byteArray : ByteArraySplitter(b, '\0')) {
+        if (byteArray.empty()) {
             continue;
         }
-        if (!recursive && (byteArray.indexOf('/') != -1)) {
+        if (!recursive && (byteArray.find('/') != std::string::npos)) {
             continue;
         }
-        files.append(QString::fromUtf8(byteArray));
+        files.append(byteArray.toString());
     }
     return files;
 }
