@@ -5,10 +5,8 @@
 */
 
 #include "replicodesettings.h"
-#include <QDebug>
 #include <QSettings>
 #include <QString>
-#include <QXmlStreamWriter>
 
 ReplicodeSettings::ReplicodeSettings(QObject *parent)
     : QObject(parent)
