@@ -30,6 +30,9 @@ Q_SIGNALS:
     void setSearchString(const QString &pattern);
     void startSearch();
     void newTab();
+    void setRegexMode(bool enabled);
+    void setCaseInsensitive(bool enabled);
+    void setExpandResults(bool enabled);
 
     //
     // KTextEditor::Command
