@@ -18,6 +18,9 @@ Results::Results(QWidget *parent)
     setupUi(this);
 
     treeView->setItemDelegate(new SearchResultsDelegate(treeView));
+    connect(treeView, &ResultsTreeView::detachClicked, this, [this] {
+        Q_EMIT requestDetachToMainWindow(this);
+    });
 
     MatchProxyModel *proxy = new MatchProxyModel(this);
     proxy->setSourceModel(&matchModel);
