@@ -23,6 +23,20 @@ public:
 
 private:
     QColor m_fg;
+    class QPushButton *const m_detachButton;
+
+protected:
+    void resizeEvent(QResizeEvent *) override;
+#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
+    void enterEvent(QEvent *event) override;
+#else
+    void enterEvent(QEnterEvent *event) override;
+#endif
+    void leaveEvent(QEvent *) override;
+
+Q_SIGNALS:
+    void geometryChanged();
+    void detachClicked();
 };
 
 #endif
