@@ -60,7 +60,7 @@ void KatePluginSymbolViewerView::parsePythonSymbols(void)
         int line = i;
         QString cl = kv->line(i);
         // concatenate continued lines and remove continuation marker
-        if (cl.length() == 0) {
+        if (cl.isEmpty()) {
             continue;
         }
         while (cl[cl.length() - 1] == QLatin1Char('\\')) {
@@ -71,6 +71,9 @@ void KatePluginSymbolViewerView::parsePythonSymbols(void)
             } else {
                 break;
             }
+            if (cl.isEmpty()) {
+                break;
+            }
         }
 
         match = class_regexp.match(cl);
