@@ -9,7 +9,6 @@
 #include "tabswitcherfilesmodel.h"
 
 #include <QBrush>
-#include <QDebug>
 #include <QFileInfo>
 #include <QMimeDatabase>
 
@@ -40,18 +39,15 @@ QString FilenameListItem::fullPath() const
 }
 
 /**
- * Note that if strs contains the empty string, the result will be ""
+ * Note that if strs contains less than 2 items, the result will be an empty string.
  */
 QString longestCommonPrefix(std::vector<QString> const &strs)
 {
-    if (strs.empty()) {
+    // only 2 or more items can have a common prefix
+    if (strs.size() < 2) {
         return QString();
     }
 
-    if (strs.size() == 1) {
-        return strs.front();
-    }
-
     // get the min length
     auto it = std::min_element(strs.begin(), strs.end(), [](const QString &lhs, const QString &rhs) {
         return lhs.size() < rhs.size();
@@ -101,6 +97,8 @@ void post_process(FilenameList &data)
         if (len > 0) { // only assign in case item.fullPath() is not empty
             // "PREFIXPATH/REMAININGPATH/BASENAME" --> "REMAININGPATH"
             item.displayPathPrefix = item.fullPath().mid(prefix_length, len);
+        } else {
+            item.displayPathPrefix.clear();
         }
     }
 }
