@@ -44,38 +44,30 @@
 // Maybe use QXmlReader to take dtds and xsds?
 
 #include "plugin_katexmlcheck.h"
-#include <QHBoxLayout>
 //#include "plugin_katexmlcheck.moc" this goes to end
 
 #include <KActionCollection>
 #include <QApplication>
 #include <QFile>
 #include <QHeaderView>
-#include <QInputDialog>
 #include <QRegExp>
 #include <QString>
 #include <QTextStream>
 #include <QTreeWidget>
 
-#include <KCursor>
 #include <KLocalizedString>
 #include <KMessageBox>
 #include <KPluginFactory>
+#include "hostprocess.h"
 #include <QAction>
 #include <QTemporaryFile>
 
-#include <QComboBox>
 #include <QFile>
-#include <QFileDialog>
-#include <QGuiApplication>
-#include <QLabel>
-#include <QLineEdit>
 #include <QPushButton>
 #include <QRegExp>
 #include <QStandardPaths>
 #include <QString>
 #include <QUrl>
-#include <QVBoxLayout>
 
 #include <ktexteditor/editor.h>
 
@@ -103,7 +95,7 @@ PluginKateXMLCheckView::PluginKateXMLCheckView(KTextEditor::Plugin *plugin, KTex
     : QObject(mainwin)
     , m_mainWindow(mainwin)
 {
-    KXMLGUIClient::setComponentName(QStringLiteral("katexmlcheck"), i18n("Kate XML check")); // where i18n resources?
+    KXMLGUIClient::setComponentName(QStringLiteral("katexmlcheck"), i18n("XML Check")); // where i18n resources?
     setXMLFile(QStringLiteral("ui.rc"));
 
     dock = m_mainWindow->createToolView(plugin,
@@ -305,9 +297,10 @@ bool PluginKateXMLCheckView::slotValidate()
     s.flush();
 
     // ensure we only execute xmllint from PATH or application package
-    QString exe = QStandardPaths::findExecutable(QStringLiteral("xmllint"));
+    static const auto executableName = QStringLiteral("xmllint");
+    QString exe = safeExecutableName(executableName);
     if (exe.isEmpty()) {
-        exe = QStandardPaths::locate(QStandardPaths::ApplicationsLocation, QStringLiteral("xmllint"));
+        exe = QStandardPaths::locate(QStandardPaths::ApplicationsLocation, executableName);
     }
     if (exe.isEmpty()) {
         KMessageBox::error(nullptr,
@@ -361,7 +354,11 @@ bool PluginKateXMLCheckView::slotValidate()
     // and needs to be ignored then):
     QRegExp re("<!--.*-->");
     re.setMinimal(true);
+#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
     text_start.remove(re);
+#else
+    text_start = re.removeIn(text_start);
+#endif
     QRegExp re_doctype("<!DOCTYPE\\s+(.*)\\s+(?:PUBLIC\\s+[\"'].*[\"']\\s+[\"'](.*)[\"']|SYSTEM\\s+[\"'](.*)[\"'])", Qt::CaseInsensitive);
     re_doctype.setMinimal(true);
 
@@ -388,7 +385,7 @@ bool PluginKateXMLCheckView::slotValidate()
     args << m_tmp_file->fileName();
     qDebug() << "m_tmp_file->fileName()=" << m_tmp_file->fileName();
 
-    m_proc.start(exe, args);
+    startHostProcess(m_proc, exe, args);
     qDebug() << "m_proc.program():" << m_proc.program(); // I want to see parameters
     qDebug() << "args=" << args;
     qDebug() << "exit code:" << m_proc.exitCode();
