@@ -38,6 +38,8 @@ private Q_SLOTS:
     void testBug460613();
     void testWindowsClosesDocuments();
     void testTabBarHidesShows();
+    void testNewWindowHasSameGlobalOptions();
+    void testBug465811();
 
 private:
     class QTemporaryDir *m_tempdir;
