@@ -22,6 +22,7 @@
 #include <KWindowConfig>
 #include <KXMLGUIFactory>
 
+#include <QApplication>
 #include <QContextMenuEvent>
 #include <QDomDocument>
 #include <QHBoxLayout>
@@ -190,7 +191,10 @@ void GUIClient::registerToolView(ToolView *tv)
     m_sidebarButtonsMenu->addAction(a);
     actionsForTool.push_back(a);
 
+    aname = QStringLiteral("kate_mdi_focus_toolview_") + tv->id;
     QAction *act = new QAction(i18n("Focus %1", tv->text), this);
+    actionCollection()->setDefaultShortcuts(act, shortcutsForActionName(aname));
+    actionCollection()->addAction(aname, act);
     connect(act, &QAction::triggered, tv, [tv = QPointer(tv)] {
         if (tv && tv->mainWindow()) {
             if (!tv->isVisible()) {
@@ -1530,6 +1534,13 @@ void MainWindow::startRestore(KConfigBase *config, const QString &group)
     KConfigGroup cg(m_restoreConfig, m_restoreGroup);
     KWindowConfig::restoreWindowSize(windowHandle(), cg);
 
+    // KWrite uses no sidebars, avoid all work beside windows sizes restoring above
+    if (KateApp::isKWrite()) {
+        m_restoreConfig = nullptr;
+        m_restoreGroup.clear();
+        return;
+    }
+
     // restore the sidebars
     for (auto &sidebar : qAsConst(m_sidebars)) {
         sidebar->startRestoreSession(cg);
@@ -1573,12 +1584,19 @@ void MainWindow::finishRestore()
 
         // Expand again to trigger splitter sync tabs/tools, but for any reason works this sometimes only after enough delay
         QTimer::singleShot(400, this, [this]() {
+            // ensure we don't steal the focus, remember old focus widget
+            QPointer<QWidget> oldFocusWidget(QApplication::focusWidget());
+
             for (auto &sidebar : m_sidebars) {
                 sidebar->updateSidebar();
             }
 
-            // ensure focus is not stolen
-            triggerFocusForCentralWidget();
+            // ensure focus is not stolen, pass back to widget or at least central area
+            if (oldFocusWidget) {
+                oldFocusWidget->setFocus();
+            } else {
+                triggerFocusForCentralWidget();
+            }
         });
     }
 
@@ -1591,6 +1609,11 @@ void MainWindow::saveSession(KConfigGroup &config)
 {
     saveMainWindowSettings(config);
 
+    // KWrite uses no sidebars, avoid all work beside windows sizes saving
+    if (KateApp::isKWrite()) {
+        return;
+    }
+
     // save main splitter sizes ;)
     config.writeEntry("Kate-MDI-H-Splitter", m_hSplitter->sizes());
     config.writeEntry("Kate-MDI-V-Splitter", m_vSplitter->sizes());
