@@ -220,7 +220,7 @@ bool KateViewSpace::eventFilter(QObject *obj, QEvent *event)
     if (button && !isActiveSpace() && event->type() == QEvent::MouseButtonPress) {
         m_viewManager->setActiveSpace(this);
         if (currentView()) {
-            m_viewManager->activateView(currentView()->document());
+            m_viewManager->activateView(currentView()->document(), this);
         }
     }
     return false;
@@ -371,7 +371,7 @@ void KateViewSpace::removeView(KTextEditor::View *v)
     // (last element could well be v->document() being removed here)
     for (auto rit = m_registeredDocuments.rbegin(); rit != m_registeredDocuments.rend(); ++rit) {
         if (auto doc = rit->doc()) {
-            m_viewManager->activateView(doc);
+            m_viewManager->activateView(doc, this);
             break;
         } else if (auto wid = rit->widget()) {
             activateWidget(wid);
@@ -459,7 +459,7 @@ void KateViewSpace::changeView(int idx)
     const auto docOrWidget = m_tabBar->tabDocument(idx);
 
     // tell the view manager to show the view
-    m_viewManager->activateView(docOrWidget);
+    m_viewManager->activateView(docOrWidget, this);
 }
 
 KTextEditor::View *KateViewSpace::currentView()
@@ -484,7 +484,7 @@ void KateViewSpace::makeActive(bool focusCurrentView)
     if (!isActiveSpace()) {
         m_viewManager->setActiveSpace(this);
         if (focusCurrentView && currentView()) {
-            m_viewManager->activateView(currentView()->document());
+            m_viewManager->activateView(currentView()->document(), this);
         }
     }
     Q_ASSERT(isActiveSpace());
@@ -609,7 +609,7 @@ void KateViewSpace::dropEvent(QDropEvent *e)
     auto droppedData = TabMimeData::data(e->mimeData());
     if (droppedData.has_value()) {
         auto doc = KateApp::self()->documentManager()->openUrl(droppedData.value().url);
-        auto view = m_viewManager->activateView(doc);
+        auto view = m_viewManager->activateView(doc, this);
         if (view) {
             view->setCursorPosition({droppedData.value().line, droppedData.value().col});
             m_dropIndicator.reset();
@@ -739,40 +739,49 @@ void KateViewSpace::closeTabRequest(int idx)
             Q_ASSERT(false);
             return;
         }
+        removeWidget(widget);
+        return;
+    }
 
-        bool shouldClose = true;
-        QMetaObject::invokeMethod(widget, "shouldClose", Q_RETURN_ARG(bool, shouldClose));
-        if (shouldClose) {
-            stack->removeWidget(widget);
-            m_registeredDocuments.removeOne(widget);
+    closeDocument(doc);
+}
 
-            m_tabBar->blockSignals(true);
-            m_tabBar->removeDocument(docOrWidget);
-            m_tabBar->blockSignals(false);
+void KateViewSpace::removeWidget(QWidget *w)
+{
+    bool shouldClose = true;
+    QMetaObject::invokeMethod(w, "shouldClose", Q_RETURN_ARG(bool, shouldClose));
+    if (shouldClose) {
+        stack->removeWidget(w);
+        m_registeredDocuments.removeOne(w);
 
-            widget->deleteLater();
-            Q_EMIT m_viewManager->mainWindow()->widgetRemoved(widget);
+        DocOrWidget widget(w);
+        const auto idx = m_tabBar->documentIdx(widget);
+        m_tabBar->blockSignals(true);
+        m_tabBar->removeDocument(widget);
+        m_tabBar->blockSignals(false);
 
+        w->deleteLater();
+        Q_EMIT m_viewManager->mainWindow()->widgetRemoved(w);
+
+        // If some tab was removed, switch to most recently used doc
+        if (idx >= 0) {
             // switch to most recently used doc
             for (auto rit = m_registeredDocuments.rbegin(); rit != m_registeredDocuments.rend(); ++rit) {
                 if (auto doc = rit->doc()) {
-                    m_viewManager->activateView(doc);
+                    m_viewManager->activateView(doc, this);
                     break;
                 } else if (auto wid = rit->widget()) {
                     activateWidget(wid);
                     break;
                 }
             }
-
-            // if this was the last doc, let viewManager know we are empty
-            if (m_registeredDocuments.isEmpty() && m_tabBar->count() == 0) {
-                Q_EMIT viewSpaceEmptied(this);
-            }
         }
-        return;
-    }
 
-    closeDocument(doc);
+        // if this was the last doc, let viewManager know we are empty
+        if (m_registeredDocuments.isEmpty() && m_tabBar->count() == 0) {
+            Q_EMIT viewSpaceEmptied(this);
+        }
+    }
 }
 
 void KateViewSpace::createNewDocument()
@@ -786,7 +795,7 @@ void KateViewSpace::createNewDocument()
     KTextEditor::Document *doc = KateApp::self()->documentManager()->createDoc();
 
     // tell the view manager to show the document
-    m_viewManager->activateView(doc);
+    m_viewManager->activateView(doc, this);
 }
 
 void KateViewSpace::focusPrevTab()
@@ -845,17 +854,12 @@ QWidgetList KateViewSpace::widgets() const
 
 bool KateViewSpace::closeTabWithWidget(QWidget *widget)
 {
-    if (!widget) {
+    int index = m_registeredDocuments.indexOf(widget);
+    if (index < 0) {
         return false;
     }
-
-    for (int i = 0; i < m_tabBar->count(); ++i) {
-        if (m_tabBar->tabData(i).value<DocOrWidget>().widget() == widget) {
-            closeTabRequest(i);
-            return true;
-        }
-    }
-    return false;
+    removeWidget(widget);
+    return true;
 }
 
 bool KateViewSpace::activateWidget(QWidget *widget)
@@ -865,17 +869,10 @@ bool KateViewSpace::activateWidget(QWidget *widget)
     }
 
     stack->setCurrentWidget(widget);
-    for (int i = 0; i < m_tabBar->count(); ++i) {
-        if (m_tabBar->tabData(i).value<DocOrWidget>().widget() == widget) {
-            m_registeredDocuments.removeOne(widget);
-            m_registeredDocuments.append(widget);
-            // "activation signal, ViewManager::viewChanged" will be emitted
-            // when tab changes
-            m_tabBar->setCurrentIndex(i);
-            return true;
-        }
-    }
-    return false;
+    m_registeredDocuments.removeOne(widget);
+    m_registeredDocuments.append(widget);
+    m_tabBar->setCurrentDocument(DocOrWidget(widget));
+    return true;
 }
 
 void KateViewSpace::focusNavigationBar()
