@@ -15,25 +15,16 @@
 #include <ktexteditor/document.h>
 #include <ktexteditor/view.h>
 
-#include <KAboutData>
-#include <KActionCollection>
-#include <KConfigGroup>
 #include <KLocalizedString>
 #include <KPluginFactory>
 #include <KSharedConfig>
 
-#include <QBoxLayout>
 #include <QCoreApplication>
 #include <QEvent>
-#include <QFileInfo>
-#include <QHeaderView>
-#include <QLabel>
 #include <QPainter>
 #include <QPointer>
 #include <QSortFilterProxyModel>
-#include <QStandardItemModel>
 #include <QStyledItemDelegate>
-#include <QTextDocument>
 #include <QTreeView>
 
 #include <drawing_utils.h>
@@ -61,12 +52,16 @@ protected:
         return l < r;
     }
 
-    bool filterAcceptsRow(int sourceRow, const QModelIndex &) const override
+    bool filterAcceptsRow(int sourceRow, const QModelIndex &parent) const override
     {
         if (pattern.isEmpty()) {
             return true;
         }
 
+        if (filterMode == Wildcard) {
+            return QSortFilterProxyModel::filterAcceptsRow(sourceRow, parent);
+        }
+
         auto sm = static_cast<KateQuickOpenModel *>(sourceModel());
         if (!sm->isValid(sourceRow)) {
             return false;
@@ -105,8 +100,8 @@ protected:
         }
 
         if (res) {
-            // +1 point for opened files
-            score += (sm->isOpened(sourceRow));
+            // extra points for opened files
+            score += (sm->isOpened(sourceRow)) * (score / 6);
 
             // extra points if file exists in project root
             // This gives priority to the files at the root
@@ -117,6 +112,8 @@ protected:
                 score += (sm->idxToFilePath(sourceRow) == name) * name.size();
             }
         }
+        //         if (res && pattern == QStringLiteral(""))
+        //             qDebug() << score << ", " << name << "==================== END\n";
 
         sm->setScoreForIndex(sourceRow, score);
 
@@ -132,6 +129,12 @@ public Q_SLOTS:
             return false;
         }
 
+        if (filterMode == Wildcard) {
+            pattern = splitted;
+            setFilterWildcard(pattern);
+            return true;
+        }
+
         beginResetModel();
         pattern = splitted;
         matchPath = pattern.contains(QLatin1Char('/'));
@@ -140,6 +143,16 @@ public Q_SLOTS:
         return true;
     }
 
+    void setFilterMode(FilterMode m)
+    {
+        beginResetModel();
+        filterMode = m;
+        if (!pattern.isEmpty() && m == Wildcard) {
+            setFilterWildcard(pattern);
+        }
+        endResetModel();
+    }
+
 private:
     static inline bool filterByPath(QStringView path, QStringView pattern, int &score)
     {
@@ -154,6 +167,7 @@ private:
 private:
     QString pattern;
     bool matchPath = false;
+    FilterMode filterMode = Fuzzy;
 };
 
 class QuickOpenStyleDelegate : public QStyledItemDelegate
@@ -166,6 +180,11 @@ public:
 
     void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const override
     {
+        if (m_filterMode != Fuzzy) {
+            QStyledItemDelegate::paint(painter, option, index);
+            return;
+        }
+
         QStyleOptionViewItem options = option;
         initStyleOption(&options, index);
 
@@ -235,8 +254,14 @@ public Q_SLOTS:
         m_filterString = text;
     }
 
+    void setFilterMode(FilterMode m)
+    {
+        m_filterMode = m;
+    }
+
 private:
     QString m_filterString;
+    FilterMode m_filterMode;
 };
 
 Q_DECLARE_METATYPE(QPointer<KTextEditor::Document>)
@@ -261,45 +286,39 @@ KateQuickOpen::KateQuickOpen(KateMainWindow *mainWindow)
     m_listView->setTextElideMode(Qt::ElideLeft);
     m_listView->setUniformRowHeights(true);
 
-    m_base_model = new KateQuickOpenModel(this);
-
-    m_model = new QuickOpenFilterProxyModel(this);
-    m_model->setFilterRole(Qt::DisplayRole);
-    m_model->setSortRole(KateQuickOpenModel::Score);
-    m_model->setFilterCaseSensitivity(Qt::CaseInsensitive);
-    m_model->setSortCaseSensitivity(Qt::CaseInsensitive);
-    m_model->setFilterKeyColumn(Qt::DisplayRole);
-
+    m_model = new KateQuickOpenModel(this);
     m_styleDelegate = new QuickOpenStyleDelegate(this);
     m_listView->setItemDelegate(m_styleDelegate);
 
-    connect(m_inputLine, &QuickOpenLineEdit::textChanged, m_styleDelegate, &QuickOpenStyleDelegate::setFilterString);
-    connect(m_inputLine, &QuickOpenLineEdit::textChanged, this, [this](const QString &text) {
-        if (m_model->setFilterText(text)) {
-            m_styleDelegate->setFilterString(text);
-            m_listView->viewport()->update();
-            reselectFirst(); // hacky way
-        }
-    });
+    connect(m_inputLine, &QuickOpenLineEdit::textChanged, this, &KateQuickOpen::reselectFirst, Qt::QueuedConnection);
     connect(m_inputLine, &QuickOpenLineEdit::returnPressed, this, &KateQuickOpen::slotReturnPressed);
     connect(m_inputLine, &QuickOpenLineEdit::listModeChanged, this, &KateQuickOpen::slotListModeChanged);
+    connect(m_inputLine, &QuickOpenLineEdit::filterModeChanged, this, &KateQuickOpen::setFilterMode);
 
     connect(m_listView, &QTreeView::activated, this, &KateQuickOpen::slotReturnPressed);
     connect(m_listView, &QTreeView::clicked, this, &KateQuickOpen::slotReturnPressed); // for single click
 
-    m_listView->setModel(m_model);
-    m_listView->setSortingEnabled(true);
-    m_model->setSourceModel(m_base_model);
-
     m_inputLine->installEventFilter(this);
     m_listView->installEventFilter(this);
     m_listView->setHeaderHidden(true);
     m_listView->setRootIsDecorated(false);
     m_listView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
 
+    m_proxyModel = new QuickOpenFilterProxyModel(this);
+    m_proxyModel->setFilterCaseSensitivity(Qt::CaseInsensitive);
+    m_proxyModel->setSourceModel(m_model);
+    m_listView->setModel(m_proxyModel);
+    connect(m_inputLine, &QuickOpenLineEdit::textChanged, m_proxyModel, [this](const QString &text) {
+        if (m_proxyModel->setFilterText(text)) {
+            m_styleDelegate->setFilterString(text);
+            m_listView->viewport()->update();
+        }
+    });
+
     setHidden(true);
 
-    m_base_model->setListMode(m_inputLine->listMode());
+    setFilterMode();
+    m_model->setListMode(m_inputLine->listMode());
 
     // fill stuff
     updateState();
@@ -334,17 +353,18 @@ bool KateQuickOpen::eventFilter(QObject *obj, QEvent *event)
 void KateQuickOpen::reselectFirst()
 {
     int first = 0;
-    if (m_mainWindow->viewManager()->views().size() > 1 && m_model->rowCount() > 1 && m_inputLine->text().isEmpty()) {
+    const auto *model = m_listView->model();
+    if (m_mainWindow->viewManager()->views().size() > 1 && model->rowCount() > 1 && m_inputLine->text().isEmpty()) {
         first = 1;
     }
 
-    QModelIndex index = m_model->index(first, 0);
+    QModelIndex index = model->index(first, 0);
     m_listView->setCurrentIndex(index);
 }
 
 void KateQuickOpen::updateState()
 {
-    m_base_model->refresh(m_mainWindow);
+    m_model->refresh(m_mainWindow);
     reselectFirst();
 
     updateViewGeometry();
@@ -355,11 +375,9 @@ void KateQuickOpen::updateState()
 void KateQuickOpen::slotReturnPressed()
 {
     // save current position before opening new url for location history
-    KateViewManager *vm = m_mainWindow->viewManager();
-    if (vm) {
-        if (KTextEditor::View *v = vm->activeView()) {
-            vm->addPositionToHistory(v->document()->url(), v->cursorPosition());
-        }
+    auto vm = m_mainWindow->viewManager();
+    if (auto v = vm->activeView()) {
+        vm->addPositionToHistory(v->document()->url(), v->cursorPosition());
     }
 
     // either get view via document pointer or url
@@ -408,11 +426,20 @@ void KateQuickOpen::slotReturnPressed()
 
 void KateQuickOpen::slotListModeChanged(KateQuickOpenModel::List mode)
 {
-    m_base_model->setListMode(mode);
+    m_model->setListMode(mode);
     // this changes things again, needs refresh, let's go all the way
     updateState();
 }
 
+void KateQuickOpen::setFilterMode()
+{
+    auto newMode = m_inputLine->filterMode();
+    m_proxyModel->setFilterMode(newMode);
+    m_listView->setSortingEnabled(newMode == Fuzzy);
+    m_styleDelegate->setFilterMode(newMode);
+    m_listView->viewport()->update();
+}
+
 void KateQuickOpen::updateViewGeometry()
 {
     const QSize centralSize = m_mainWindow->size();
